@@ -1,6 +1,6 @@
 /*
  *	cook - file construction tool
- *	Copyright (C) 1991, 1992, 1993, 1994 Peter Miller.
+ *	Copyright (C) 1991, 1992, 1993, 1994, 1995, 1997, 1998 Peter Miller;
  *	All rights reserved.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -15,7 +15,7 @@
  *
  *	You should have received a copy of the GNU General Public License
  *	along with this program; if not, write to the Free Software
- *	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
  *
  * MANIFEST: operating system entry point, and command line argument parsing
  */
@@ -27,12 +27,13 @@
 
 #include <arglex.h>
 #include <cache.h>
-#include <error.h>
+#include <error_intl.h>
 #include <help.h>
-#include <main.h>
+#include <lang.h>
+#include <progname.h>
 #include <sniff.h>
 #include <str.h>
-#include <trace.h>
+#include <verbose.h>
 #include <version.h>
 
 
@@ -41,6 +42,9 @@ static void usage _((void));
 static void
 usage()
 {
+	char		*progname;
+
+	progname = progname_get();
 	fprintf(stderr, "usage: %s [ <option>... ] <filename>\n", progname);
 	fprintf(stderr, "       %s -Help\n", progname);
 	fprintf(stderr, "       %s -VERsion\n", progname);
@@ -48,22 +52,6 @@ usage()
 }
 
 
-static void c_incl_help _((void));
-
-static void
-c_incl_help()
-{
-	static char *text[] =
-	{
-#include <../man1/c_incl.h>
-	};
-
-	trace(("c_incl_help()\n{\n"/*}*/));
-	help(text, SIZEOF(text), usage);
-	trace((/*{*/"}\n"));
-}
-
-
 enum
 {
 	arglex_token_absent_local_error,
@@ -74,14 +62,24 @@ enum
 	arglex_token_absent_system_error,
 	arglex_token_absent_system_ignore,
 	arglex_token_absent_system_mention,
+	arglex_token_escape_newline,
 	arglex_token_include,
-	arglex_token_no_cache,
-	arglex_token_no_system,
+	arglex_token_language,
 	arglex_token_lang_c,
 	arglex_token_lang_roff,
+	arglex_token_absolute,
+	arglex_token_no_absolute,
+	arglex_token_cache,
+	arglex_token_cache_not,
+	arglex_token_no_source_relative_includes,
+	arglex_token_no_system,
+	arglex_token_output,
 	arglex_token_prefix,
-	arglex_token_suffix,
-	arglex_token_verbose
+	arglex_token_recursive,
+	arglex_token_recursive_not,
+	arglex_token_remove_leading_path,
+	arglex_token_substitute_leading_path,
+	arglex_token_suffix
 };
 
 static	arglex_table_ty	argtab[] =
@@ -90,6 +88,10 @@ static	arglex_table_ty	argtab[] =
 		"-Absent",
 		(arglex_token_ty)arglex_token_absent_local_ignore,
 	},
+	{
+		"-Absolute_Paths",
+		(arglex_token_ty)arglex_token_absolute,
+	},
 	{
 		"-Absent_Local_Error",
 		(arglex_token_ty)arglex_token_absent_local_error,
@@ -130,6 +132,10 @@ static	arglex_table_ty	argtab[] =
 		"-Empty_If_Absent",
 		(arglex_token_ty)arglex_token_absent_program_ignore,
 	},
+	{
+		"-Escape_Newlines",
+		(arglex_token_ty)arglex_token_escape_newline,
+	},
 	{
 		"-\\I*",
 		(arglex_token_ty)arglex_token_include,
@@ -138,18 +144,50 @@ static	arglex_table_ty	argtab[] =
 		"-Include",
 		(arglex_token_ty)arglex_token_include,
 	},
+	{
+		"-Language",
+		(arglex_token_ty)arglex_token_language,
+	},
+	{
+		"-No_Absolute_Paths",
+		(arglex_token_ty)arglex_token_no_absolute,
+	},
+	{
+		"-CAche",
+		(arglex_token_ty)arglex_token_cache,
+	},
 	{
 		"-No_Cache",
-		(arglex_token_ty)arglex_token_no_cache,
+		(arglex_token_ty)arglex_token_cache_not,
+	},
+	{
+		"-No_Source_Relative_Includes",
+		(arglex_token_ty)arglex_token_no_source_relative_includes,
 	},
 	{
 		"-No_System",
 		(arglex_token_ty)arglex_token_no_system,
 	},
+	{
+		"-Output",
+		(arglex_token_ty)arglex_token_output,
+	},
 	{
 		"-PREfix",
 		(arglex_token_ty)arglex_token_prefix,
 	},
+	{
+		"-RECursion",
+		(arglex_token_ty)arglex_token_recursive,
+	},
+	{
+		"-No_RECursion",
+		(arglex_token_ty)arglex_token_recursive_not,
+	},
+	{
+		"-Remove_Leading_Path",
+		(arglex_token_ty)arglex_token_remove_leading_path,
+	},
 	{
 		"-Roff",
 		(arglex_token_ty)arglex_token_lang_roff,
@@ -159,12 +197,12 @@ static	arglex_table_ty	argtab[] =
 		(arglex_token_ty)arglex_token_absent_system_ignore,
 	},
 	{
-		"-SUFfix",
-		(arglex_token_ty)arglex_token_suffix,
+		"-Substitute_Leading_Path",
+		(arglex_token_ty)arglex_token_substitute_leading_path,
 	},
 	{
-		"-Verbose",
-		(arglex_token_ty)arglex_token_verbose,
+		"-SUFfix",
+		(arglex_token_ty)arglex_token_suffix,
 	},
 
 	/* end marker */
@@ -181,8 +219,11 @@ main(argc, argv)
 {
 	char		*source;
 	int		no_system;
-	int		no_cache;
+	int		cache;
 	sniff_ty	*language;
+	char		*arg1;
+	char		*arg2;
+	sub_context_ty	*scp;
 
 	arglex_init(argc, argv, argtab);
 	str_initialize();
@@ -190,7 +231,7 @@ main(argc, argv)
 	switch (arglex())
 	{
 	case arglex_token_help:
-		c_incl_help();
+		help((char *)0, usage);
 		exit(0);
 
 	case arglex_token_version:
@@ -203,45 +244,54 @@ main(argc, argv)
 
 	source = 0;
 	no_system = 0;
-	no_cache = 0;
+	cache = -1;
 	language = 0;
 	option.o_absent_local = -1;
 	option.o_absent_system = -1;
 	option.o_absent_program = -1;
+	option.absolute = -1;
+	option.recursive = -1;
 	while (arglex_token != arglex_token_eoln)
 	{
 		switch(arglex_token)
 		{
 		default:
-			error
-			(
-				"misplaced \"%s\" command line argument",
-				arglex_value.alv_string
-			);
-			usage();
+			generic_argument(usage);
+			continue;
 
 		case arglex_token_string:
 			if (source)
-				fatal("too many filenames specified");
-			source = arglex_value.alv_string;
-			break;
-
-		case arglex_token_verbose:
-			if (option.o_verbose)
 			{
-				duplicate:
-				fatal
+				error_intl
 				(
-					"duplicate \"%s\" option",
-					arglex_value.alv_string
+					0,
+					i18n("too many filenames specified")
 				);
+				usage();
 			}
-			++option.o_verbose;
+			source = arglex_value.alv_string;
 			break;
 
 		case arglex_token_absent_local_ignore:
 			if (option.o_absent_local != -1)
-				goto duplicate;
+			{
+				duplicate:
+				scp = sub_context_new();
+				sub_var_set
+				(
+					scp,
+					"Name",
+					"%s",
+					arglex_value.alv_string
+				);
+				error_intl
+				(
+					scp,
+					i18n("duplicate \"$name\" option")
+				);
+				sub_context_delete(scp);
+				usage();
+			}
 			option.o_absent_local = absent_ignore;
 			break;
 
@@ -288,9 +338,29 @@ main(argc, argv)
 			break;
 
 		case arglex_token_include:
-			if (arglex() != arglex_token_string)
-				fatal("-Include requires a string argument");
-			sniff_include(arglex_value.alv_string);
+			switch (arglex())
+			{
+			default:
+				scp = sub_context_new();
+				sub_var_set
+				(
+					scp,
+					"Name",
+					"%s",
+					arglex_token_name(arglex_token_include)
+				);
+				error_intl(scp, i18n("$name needs string"));
+				sub_context_delete(scp);
+				usage();
+
+			case arglex_token_string:
+				sniff_include(arglex_value.alv_string);
+				break;
+
+			case arglex_token_stdio:
+				sniff_include_cut();
+				break;
+			}
 			break;
 
 		case arglex_token_no_system:
@@ -299,52 +369,121 @@ main(argc, argv)
 			no_system++;
 			break;
 
-		case arglex_token_no_cache:
-			if (no_cache)
+		case arglex_token_cache:
+			if (cache >= 0)
 				goto duplicate;
-			no_cache++;
+			cache = 1;
+			break;
+
+		case arglex_token_cache_not:
+			if (cache >= 0)
+				goto duplicate;
+			cache = 0;
+			break;
+
+		case arglex_token_no_source_relative_includes:
+			if (option.no_src_rel_inc)
+				goto duplicate;
+			option.no_src_rel_inc++;
+			break;
+
+		case arglex_token_language:
+			if (language)
+				goto duplicate;
+			if (arglex() != arglex_token_string)
+			{
+				scp = sub_context_new();
+				sub_var_set
+				(
+					scp,
+					"Name",
+					"%s",
+					arglex_token_name(arglex_token_language)
+				);
+				error_intl(scp, i18n("$name needs string"));
+				sub_context_delete(scp);
+				usage();
+			}
+			language = lang_from_name(arglex_value.alv_string);
 			break;
 
 		case arglex_token_lang_c:
 			if (language)
 				goto duplicate;
-			language = &lang_c;
+			language = lang_from_name("c");
 			break;
 
 		case arglex_token_lang_roff:
 			if (language)
 				goto duplicate;
-			language = &lang_roff;
+			language = lang_from_name("roff");
 			break;
 		
 		case arglex_token_prefix:
-			if (prefix)
-				goto duplicate;
 			if (arglex() != arglex_token_string)
 				usage();
-			prefix = arglex_value.alv_string;
+			sniff_prefix_set(arglex_value.alv_string);
 			break;
 		
 		case arglex_token_suffix:
-			if (suffix)
+			if (arglex() != arglex_token_string)
+				usage();
+			sniff_suffix_set(arglex_value.alv_string);
+			break;
+
+		case arglex_token_output:
+			if (option.output)
 				goto duplicate;
 			if (arglex() != arglex_token_string)
 				usage();
-			suffix = arglex_value.alv_string;
+			option.output = arglex_value.alv_string;
 			break;
 
-#ifdef DEBUG
-		case arglex_token_tracing:
+		case arglex_token_remove_leading_path:
 			if (arglex() != arglex_token_string)
-				fatal("-TRACIng requires one or more string arguments");
-			for (;;)
-			{
-				trace_enable(arglex_value.alv_string);
-				if (arglex() != arglex_token_string)
-					break;
-			}
-			continue;
-#endif
+				usage();
+			sniff_remove_leading_path(arglex_value.alv_string);
+			break;
+
+		case arglex_token_substitute_leading_path:
+			if (arglex() != arglex_token_string)
+				usage();
+			arg1 = arglex_value.alv_string;
+			if (arglex() != arglex_token_string)
+				usage();
+			arg2 = arglex_value.alv_string;
+			sniff_substitute_leading_path(arg1, arg2);
+			break;
+
+		case arglex_token_absolute:
+			if (option.absolute != -1)
+				goto duplicate;
+			option.absolute = 1;
+			break;
+
+		case arglex_token_no_absolute:
+			if (option.absolute != -1)
+				goto duplicate;
+			option.absolute = 0;
+			break;
+
+		case arglex_token_recursive:
+			if (option.recursive != -1)
+				goto duplicate;
+			option.recursive = 1;
+			break;
+
+		case arglex_token_recursive_not:
+			if (option.recursive != -1)
+				goto duplicate;
+			option.recursive = 0;
+			break;
+
+		case arglex_token_escape_newline:
+			if (option.escape_newline)
+				goto duplicate;
+			option.escape_newline = 1;
+			break;
 		}
 		arglex();
 	}
@@ -355,13 +494,20 @@ main(argc, argv)
 	if (option.o_absent_program == -1)
 		option.o_absent_program = absent_error;
 	if (!source)
-		fatal("no input file specified");
+	{
+		error_intl(0, i18n("no input file specified"));
+		usage();
+	}
+	if (option.absolute < 0)
+		option.absolute = !no_system;
+	if (option.recursive == 0 && cache < 0)
+		cache = 0;
 
 	/*
 	 * set the language to be used
 	 */
 	if (!language)
-		language = &lang_c;
+		language = lang_from_name("c");
 	sniff_language(language);
 
 	/*
@@ -374,10 +520,10 @@ main(argc, argv)
 	/*
 	 * read and analyze the file
 	 */
-	if (!no_cache)
+	if (cache)
 		cache_read();
 	sniff(source);
-	if (!no_cache)
+	if (cache)
 		cache_write();
 	exit(0);
 	return 0;
