@@ -1,6 +1,6 @@
 /*
  *	cook - file construction tool
- *	Copyright (C) 1994 Peter Miller.
+ *	Copyright (C) 1994, 1997, 1998 Peter Miller;
  *	All rights reserved.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -15,32 +15,31 @@
  *
  *	You should have received a copy of the GNU General Public License
  *	along with this program; if not, write to the Free Software
- *	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
  *
- * MANIFEST: isolate dirent.h/ndir.h differences
+ * MANIFEST: insulate against <dirent.h> vs <ndir.h> differences
  */
 
 #ifndef COMMON_AC_DIRENT_H
 #define COMMON_AC_DIRENT_H
 
-#include <ac/unistd.h>
-		
-/* unistd.h defines _POSIX_VERSION on POSIX.1 systems.  */
-#if defined(DIRENT) || defined(_POSIX_VERSION)
+#include <config.h>
+
+#ifdef HAVE_DIRENT_H
 #include <dirent.h>
 #define NLENGTH(dirent) (strlen((dirent)->d_name))
-#else /* not (DIRENT or _POSIX_VERSION) */
+#else /* not dirent.h */
 #define dirent direct
 #define NLENGTH(dirent) ((dirent)->d_namlen)
-#ifdef SYSNDIR
+#ifdef HAVE_SYS_NDIR_H
 #include <sys/ndir.h>
 #endif /* SYSNDIR */
-#ifdef SYSDIR
+#ifdef HAVE_SYS_DIR_H
 #include <sys/dir.h>
 #endif /* SYSDIR */
-#ifdef NDIR
+#ifdef HAVE_NDIR_H
 #include <ndir.h>
 #endif /* NDIR */
-#endif /* not (DIRENT or _POSIX_VERSION) */
+#endif /* not dirent.h */
 
 #endif /* COMMON_AC_DIRENT_H */
