@@ -1,6 +1,6 @@
 /*
  *	cook - file construction tool
- *	Copyright (C) 1990, 1991, 1992, 1993, 1994 Peter Miller.
+ *	Copyright (C) 1990, 1991, 1992, 1993, 1994, 1997 Peter Miller;
  *	All rights reserved.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -15,39 +15,27 @@
  *
  *	You should have received a copy of the GNU General Public License
  *	along with this program; if not, write to the Free Software
- *	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
  *
  * MANIFEST: interface definition for common/error.c
  */
 
-#ifndef ERROR_H
-#define ERROR_H
+#ifndef COMMON_ERROR_H
+#define COMMON_ERROR_H
 
 #include <main.h>
 
-void error _((char *, ...));
-void fatal _((char *, ...));
-void nerror _((char *, ...));
-void nfatal _((char *, ...));
+/*
+ * These were the old names.  Use *_intl instead.
+ */
+#define error +/+/+/+
+#define fatal +/+/+/+
+#define nerror +/+/+/+
+#define nfatal +/+/+/+
 
-#ifdef DEBUG
-# if defined(__STDC__) || defined(__stdc__)
-#  define assert(c) cook_assert((c) != 0, #c, __FILE__, __LINE__)
-# else
-#  define assert(c) cook_assert((c) != 0, "c", __FILE__, __LINE__)
-# endif
-void cook_assert _((int, char *, char *, int));
-#else
-# define assert(c)
-#endif
+void error_raw _((char *, ...));
+void fatal_raw _((char *, ...));
+void nerror_raw _((char *, ...));
+void nfatal_raw _((char *, ...));
 
-typedef void (*quit_ty)_((void));
-void quit_handler _((quit_ty));
-void quit _((int));
-
-void star _((void));
-void star_eoln _((void));
-void star_sync _((void));
-void star_enable _((void));
-
-#endif /* ERROR_H */
+#endif /* COMMON_ERROR_H */
