@@ -1,6 +1,6 @@
 /*
  *	cook - file construction tool
- *	Copyright (C) 1994 Peter Miller.
+ *	Copyright (C) 1994, 1996, 1997 Peter Miller;
  *	All rights reserved.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -15,7 +15,7 @@
  *
  *	You should have received a copy of the GNU General Public License
  *	along with this program; if not, write to the Free Software
- *	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
  *
  * MANIFEST: functions to manipulate snefru fingerprints
  *
@@ -30,9 +30,28 @@
  */
 
 #include <stdio.h>
+#include <ac/limits.h>
 
 #include <fp/snefru.h>
-#include <find_sizes.h>
+
+
+/*
+ * work out a 32-bit type
+ */
+#if (SIZEOF_SHORT * CHAR_BIT == 32)
+typedef unsigned short uint32;
+#else
+#if (SIZEOF_INT * CHAR_BIT == 32)
+typedef unsigned int uint32;
+#else
+#if (SIZEOF_LONG * CHAR_BIT == 32)
+typedef unsigned long uint32;
+#else
+#error "no type is exactly 32 bits"
+#endif
+#endif
+#endif
+
 
 /* 
  * NAME
