@@ -1,6 +1,6 @@
 /*
  *	cook - a program construction tool
- *	Copyright (C) 1991, 1992, 1993, 1994 Peter Miller.
+ *	Copyright (C) 1991, 1992, 1993, 1994, 1997 Peter Miller;
  *	All rights reserved.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -15,7 +15,7 @@
  *
  *	You should have received a copy of the GNU General Public License
  *	along with this program; if not, write to the Free Software
- *	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
  *
  * MANIFEST: interface definition for comon/help.c
  */
@@ -25,6 +25,8 @@
 
 #include <main.h>
 
-void help _((char **text, int text_lem, void (*usage)(void)));
+void help _((char *, void (*)(void)));
+void generic_argument _((void(*)(void)));
+void bad_argument _((void(*)(void)));
 
 #endif /* HELP_H */
