@@ -1,6 +1,6 @@
 /*
  *	cook - file construction tool
- *	Copyright (C) 1990, 1991, 1992, 1993, 1994 Peter Miller.
+ *	Copyright (C) 1990, 1991, 1992, 1993, 1994, 1997 Peter Miller;
  *	All rights reserved.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -15,7 +15,7 @@
  *
  *	You should have received a copy of the GNU General Public License
  *	along with this program; if not, write to the Free Software
- *	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
  *
  * MANIFEST: functions to manipulate dynamic memory
  */
@@ -25,8 +25,8 @@
 #include <ac/stdlib.h>
 #include <errno.h>
 
-#include <mem.h>
 #include <error.h>
+#include <mem.h>
 
 
 /*
@@ -50,10 +50,10 @@ static void
 memory_error()
 {
 #ifdef DEBUG
-	nerror("memory allocator");
+	nerror_raw("memory allocator");
 	abort();
 #else
-	nfatal("memory allocator");
+	nfatal_raw("memory allocator");
 #endif
 }
 
@@ -83,14 +83,10 @@ mem_alloc(n)
 
 	if (n < 1)
 		n = 1;
-	errno = 0;
+	errno = ENOMEM;
 	p = malloc(n);
 	if (!p)
-	{
-		if (!errno)
-			errno = ENOMEM;
 		memory_error();
-	}
 	return p;
 }
 
@@ -131,17 +127,13 @@ mem_change_size(p, n)
 {
 	if (n < 1)
 		n = 1;
-	errno = 0;
+	errno = ENOMEM;
 	if (!p)
 		p = malloc(n);
 	else
 		p = realloc(p, n);
 	if (!p)
-	{
-		if (!errno)
-			errno = ENOMEM;
 		memory_error();
-	}
 	return p;
 }
 
