@@ -1,6 +1,6 @@
 /*
  *	cook - file construction tool
- *	Copyright (C) 1990, 1991, 1992, 1993, 1994 Peter Miller.
+ *	Copyright (C) 1990, 1991, 1992, 1993, 1994, 1995, 1997 Peter Miller;
  *	All rights reserved.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -15,7 +15,7 @@
  *
  *	You should have received a copy of the GNU General Public License
  *	along with this program; if not, write to the Free Software
- *	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
  *
  * MANIFEST: functions to report execution traces
  */
@@ -24,11 +24,11 @@
 #include <ac/stddef.h>
 #include <stdio.h>
 #include <ac/string.h>
+#include <ac/stdarg.h>
 
 #include <error.h>
 #include <mem.h>
-#include <arglex.h>
-#include <s-v-arg.h>
+#include <progname.h>
 #include <str.h>
 #include <trace.h>
 
@@ -141,7 +141,9 @@ trace_putchar(c)
 	static char	*cp;
 	static int	in_col;
 	static int	out_col;
+	char		*progname;
 
+	progname = progname_get();
 	if (!page_width)
 	{
 		page_width = PAGE_WIDTH - 24;
@@ -174,7 +176,7 @@ trace_putchar(c)
 		fputs(buffer, stderr);
 		fflush(stderr);
 		if (ferror(stderr))
-			nfatal("(stderr)");
+			nfatal_raw("standard error");
 		cp = 0;
 		break;
 
@@ -277,6 +279,13 @@ trace_enable(file)
 	kp->flag = 3; /* enabled */
 	if (kp->flag_p)
 		*kp->flag_p = kp->flag;
+
+	/*
+	 * silence a warning
+	 */
+#ifdef DEBUG
+	trace_pretest_result = 0;
+#endif
 }
 
 
