@@ -1,6 +1,6 @@
 /*
  *	cook - file construction tool
- *	Copyright (C) 1994 Peter Miller.
+ *	Copyright (C) 1994, 1997, 1998 Peter Miller;
  *	All rights reserved.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -15,13 +15,13 @@
  *
  *	You should have received a copy of the GNU General Public License
  *	along with this program; if not, write to the Free Software
- *	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
  *
  * MANIFEST: functions to manipulate archive files
  */
 
-#include <ctype.h>
-#include <stdio.h>
+#include <ac/ctype.h>
+#include <ac/stdio.h>
 #include <errno.h>
 #include <ac/ar.h>
 #include <ac/fcntl.h>
@@ -29,11 +29,10 @@
 #include <ac/string.h>
 #include <sys/types.h>
 #include <sys/stat.h>
-#include <utime.h>
+#include <ac/utime.h>
 #include <ac/unistd.h>
 
 #include <archive.h>
-#include <error.h>
 #include <fp.h>
 #include <mem.h>
 #include <trace.h>
@@ -1489,7 +1488,7 @@ archive_stat(name, st)
 	string_ty	*path;
 	string_ty	*member;
 	archive_file_ty	*afp;
-	int		err;
+	int		errno_hold;
 	int		result;
 
 	/*
@@ -1497,6 +1496,8 @@ archive_stat(name, st)
 	 */
 	trace(("archive_stat(name = \"%s\")\n{\n"/*}*/, name->str_text));
 	result = -1;
+	path = 0;
+	member = 0;
 	if (!archive_match(name, &path, &member))
 	{
 		/*
@@ -1506,6 +1507,8 @@ archive_stat(name, st)
 		errno = ENOENT;
 		goto done;
 	}
+	assert(path);
+	assert(member);
 
 	/*
 	 * open the archive file
@@ -1525,11 +1528,11 @@ archive_stat(name, st)
 	trace(("stat\n"));
 	if (archive_file_stat(afp, member, st))
 	{
-		err = errno;
+		errno_hold = errno;
 		archive_file_close(afp);
 		str_free(member);
 		str_free(path);
-		errno = err;
+		errno = errno_hold;
 		goto done;
 	}
 
@@ -1552,10 +1555,12 @@ archive_stat(name, st)
 	 * here for all exits
 	 */
 	done:
-{int err = errno;
-	trace(("return %d; /* errno = %d */\n", result, err));
+#ifdef DEBUG
+	errno_hold = errno;
+	trace(("return %d; /* errno = %d */\n", result, errno_hold));
 	trace((/*{*/"}\n"));
-errno = err;}
+	errno = errno_hold;
+#endif
 	return result;
 }
 
@@ -1576,6 +1581,8 @@ archive_utime(name, ut)
 	 */
 	trace(("archive_utime(name = \"%s\")\n{\n"/*}*/, name->str_text));
 	result = -1;
+	path = 0;
+	member = 0;
 	if (!archive_match(name, &path, &member))
 	{
 		/*
@@ -1585,6 +1592,8 @@ archive_utime(name, ut)
 		errno = ENOENT;
 		goto done;
 	}
+	assert(path);
+	assert(member);
 
 	/*
 	 * open the archive file
@@ -1651,6 +1660,8 @@ archive_fingerprint(fp, name, buf)
 	 */
 	trace(("archive_fingerprint(name = \"%s\")\n{\n"/*}*/, name->str_text));
 	result = -1;
+	path = 0;
+	member = 0;
 	if (!archive_match(name, &path, &member))
 	{
 		/*
@@ -1660,6 +1671,8 @@ archive_fingerprint(fp, name, buf)
 		errno = ENOENT;
 		goto done;
 	}
+	assert(path);
+	assert(member);
 
 	/*
 	 * open the archive file
