@@ -1,6 +1,6 @@
 /*
  *	cook - file construction tool
- *	Copyright (C) 1994 Peter Miller.
+ *	Copyright (C) 1994, 1997, 1998 Peter Miller;
  *	All rights reserved.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -15,16 +15,24 @@
  *
  *	You should have received a copy of the GNU General Public License
  *	along with this program; if not, write to the Free Software
- *	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
  *
- * MANIFEST: interface definition for exists.h
+ * MANIFEST: interface definition for cook/builtin/exists.c
  */
 
 #ifndef COOK_BUILTIN_EXISTS_H
 #define COOK_BUILTIN_EXISTS_H
 
-#include <word.h>
+#include <main.h>
 
-int builtin_exists _((wlist *, wlist *));
+struct string_list_ty; /* existence */
+struct expr_position_ty; /* existence */
+struct opcode_context_ty; /* existence */
+
+int builtin_exists _((struct string_list_ty *, const struct string_list_ty *,
+	const struct expr_position_ty *, const struct opcode_context_ty *));
+int builtin_exists_symlink _((struct string_list_ty *,
+	const struct string_list_ty *, const struct expr_position_ty *,
+	const struct opcode_context_ty *));
 
 #endif /* COOK_BUILTIN_EXISTS_H */
