@@ -1,6 +1,6 @@
 /*
  *	cook - file construction tool
- *	Copyright (C) 1994 Peter Miller.
+ *	Copyright (C) 1994, 1997, 1998 Peter Miller;
  *	All rights reserved.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -15,38 +15,50 @@
  *
  *	You should have received a copy of the GNU General Public License
  *	along with this program; if not, write to the Free Software
- *	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
  *
  * MANIFEST: functions to implement the builtin join function
  */
 
 #include <builtin/join.h>
-#include <expr.h>
+#include <error_intl.h>
+#include <expr/position.h>
+#include <str_list.h>
+#include <trace.h>
 
 
 int
-builtin_join(result, arg)
-	wlist		*result;
-	wlist		*arg;
+builtin_join(result, arg, pp, ocp)
+	string_list_ty	*result;
+	const string_list_ty *arg;
+	const expr_position_ty *pp;
+	const struct opcode_context_ty *ocp;
 {
 	size_t		n;
 	size_t		j;
 	string_ty	*s;
 
-	if ((arg->wl_nwords & 1) == 0)
+	trace(("join\n"));
+	if ((arg->nstrings & 1) == 0)
 	{
-		expr_error
+		sub_context_ty	*scp;
+
+		scp = sub_context_new();
+		sub_var_set(scp, "Name", "%S", arg->string[0]);
+		error_with_position
 		(
-			"%s: requires an even number of arguments",
-			arg->wl_word[0]->str_text
+			pp,
+			scp,
+			i18n("$name: requires an even number of arguments")
 		);
+		sub_context_delete(scp);
 		return -1;
 	}
-	n = arg->wl_nwords / 2;
+	n = arg->nstrings / 2;
 	for (j = 1; j <= n; ++j)
 	{
-		s = str_catenate(arg->wl_word[j], arg->wl_word[j + n]);
-		wl_append(result, s);
+		s = str_catenate(arg->string[j], arg->string[j + n]);
+		string_list_append(result, s);
 		str_free(s);
 	}
 	return 0;
