@@ -1,6 +1,6 @@
 /*
  *	cook - file construction tool
- *	Copyright (C) 1994 Peter Miller.
+ *	Copyright (C) 1994, 1997 Peter Miller;
  *	All rights reserved.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -15,18 +15,25 @@
  *
  *	You should have received a copy of the GNU General Public License
  *	along with this program; if not, write to the Free Software
- *	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
  *
- * MANIFEST: interface definition for match.h
+ * MANIFEST: interface definition for cook/builtin/match.c
  */
 
 #ifndef COOK_BUILTIN_MATCH_H
 #define COOK_BUILTIN_MATCH_H
 
-#include <word.h>
+#include <main.h>
 
-int builtin_match _((wlist *, wlist *));
-int builtin_match_mask _((wlist *, wlist *));
-int builtin_fromto _((wlist *, wlist *));
+struct string_list_ty; /* existence */
+struct expr_position_ty; /* existence */
+struct opcode_context_ty; /* existence */
+
+int builtin_match _((struct string_list_ty *, const struct string_list_ty *,
+	const struct expr_position_ty *, const struct opcode_context_ty *));
+int builtin_match_mask _((struct string_list_ty *, const struct string_list_ty *,
+	const struct expr_position_ty *, const struct opcode_context_ty *));
+int builtin_fromto _((struct string_list_ty *, const struct string_list_ty *,
+	const struct expr_position_ty *, const struct opcode_context_ty *));
 
 #endif /* COOK_BUILTIN_MATCH_H */
