@@ -1,6 +1,6 @@
 /*
  *	cook - file construction tool
- *	Copyright (C) 1991, 1992, 1993, 1994 Peter Miller.
+ *	Copyright (C) 1991, 1992, 1993, 1994, 1997, 1998 Peter Miller;
  *	All rights reserved.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -15,7 +15,7 @@
  *
  *	You should have received a copy of the GNU General Public License
  *	along with this program; if not, write to the Free Software
- *	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
  *
  * MANIFEST: functions to implement the builtin pathname functions
  *
@@ -27,9 +27,10 @@
  */
 
 #include <builtin/pathname.h>
-#include <error.h>
-#include <expr.h>
+#include <error_intl.h>
+#include <expr/position.h>
 #include <os.h>
+#include <trace.h>
 
 
 /*
@@ -37,7 +38,7 @@
  *	builtin_dir - dir part
  *
  * SYNOPSIS
- *	int builtin_dir(wlist *result, wlist *args);
+ *	int builtin_dir(string_list_ty *result, string_list_ty *args);
  *
  * DESCRIPTION
  *	"dir" is a built-in function of cook, described as follows:
@@ -51,36 +52,45 @@
  * CAVEAT
  *	The returned result is in dynamic memory.
  *	It is the responsibility of the caller to dispose of
- *	the result when it is finished, with a wl_free() call.
+ *	the result when it is finished, with a string_list_destructor() call.
  */
 
 int
-builtin_dir(result, args)
-	wlist		*result;
-	wlist		*args;
+builtin_dir(result, args, pp, ocp)
+	string_list_ty	*result;
+	const string_list_ty *args;
+	const expr_position_ty *pp;
+	const struct opcode_context_ty *ocp;
 {
-	int		j;
+	size_t		j;
 
+	trace(("dir\n"));
 	assert(result);
 	assert(args);
-	assert(args->wl_nwords);
-	if (args->wl_nwords < 2)
+	assert(args->nstrings);
+	if (args->nstrings < 2)
 	{
-		expr_error
+		sub_context_ty	*scp;
+
+		scp = sub_context_new();
+		sub_var_set(scp, "Name", "%S", args->string[0]);
+		error_with_position
 		(
-			"%s: requires one or more arguments",
-			args->wl_word[0]->str_text
+			pp,
+			scp,
+			i18n("$name: requires one or more arguments")
 		);
+		sub_context_delete(scp);
 		return -1;
 	}
-	for (j = 1; j < args->wl_nwords; j++)
+	for (j = 1; j < args->nstrings; j++)
 	{
 		string_ty	*s;
 
-		s = os_dirname(args->wl_word[j]);
+		s = os_dirname(args->string[j]);
 		if (!s)
 			return -1;
-		wl_append(result, s);
+		string_list_append(result, s);
 		str_free(s);
 	}
 	return 0;
@@ -92,7 +102,7 @@ builtin_dir(result, args)
  *	builtin_entryname - entryname part
  *
  * SYNOPSIS
- *	int builtin_entryname(wlist *result, wlist *args);
+ *	int builtin_entryname(string_list_ty *result, string_list_ty *args);
  *
  * DESCRIPTION
  *	Defined is a built-in function of cook, described as follows:
@@ -106,36 +116,45 @@ builtin_dir(result, args)
  * CAVEAT
  *	The returned result is in dynamic memory.
  *	It is the responsibility of the caller to dispose of
- *	the result when it is finished, with a wl_free() call.
+ *	the result when it is finished, with a string_list_destructor() call.
  */
 
 int
-builtin_entryname(result, args)
-	wlist		*result;
-	wlist		*args;
+builtin_entryname(result, args, pp, ocp)
+	string_list_ty	*result;
+	const string_list_ty *args;
+	const expr_position_ty *pp;
+	const struct opcode_context_ty *ocp;
 {
-	int		j;
+	size_t		j;
 
+	trace(("entryname\n"));
 	assert(result);
 	assert(args);
-	assert(args->wl_nwords);
-	if (args->wl_nwords < 2)
+	assert(args->nstrings);
+	if (args->nstrings < 2)
 	{
-		expr_error
+		sub_context_ty	*scp;
+
+		scp = sub_context_new();
+		sub_var_set(scp, "Name", "%S", args->string[0]);
+		error_with_position
 		(
-			"%s: requires one or more arguments",
-			args->wl_word[0]->str_text
+			pp,
+			scp,
+			i18n("$name: requires one or more arguments")
 		);
+		sub_context_delete(scp);
 		return -1;
 	}
-	for (j = 1; j < args->wl_nwords; j++)
+	for (j = 1; j < args->nstrings; j++)
 	{
 		string_ty	*s;
 
-		s = os_entryname(args->wl_word[j]);
+		s = os_entryname(args->string[j]);
 		if (!s)
 			return -1;
-		wl_append(result, s);
+		string_list_append(result, s);
 		str_free(s);
 	}
 	return 0;
@@ -147,7 +166,7 @@ builtin_entryname(result, args)
  *	builtin_pathname - pathname part
  *
  * SYNOPSIS
- *	int builtin_pathname(wlist *result, wlist *args);
+ *	int builtin_pathname(string_list_ty *result, string_list_ty *args);
  *
  * DESCRIPTION
  *	Defined is a built-in function of cook, described as follows:
@@ -161,36 +180,45 @@ builtin_entryname(result, args)
  * CAVEAT
  *	The returned result is in dynamic memory.
  *	It is the responsibility of the caller to dispose of
- *	the result when it is finished, with a wl_free() call.
+ *	the result when it is finished, with a string_list_destructor() call.
  */
 
 int
-builtin_pathname(result, args)
-	wlist		*result;
-	wlist		*args;
+builtin_pathname(result, args, pp, ocp)
+	string_list_ty	*result;
+	const string_list_ty *args;
+	const expr_position_ty *pp;
+	const struct opcode_context_ty *ocp;
 {
-	int		j;
+	size_t		j;
 
+	trace(("pathname\n"));
 	assert(result);
 	assert(args);
-	assert(args->wl_nwords);
-	if (args->wl_nwords < 2)
+	assert(args->nstrings);
+	if (args->nstrings < 2)
 	{
-		expr_error
+		sub_context_ty	*scp;
+
+		scp = sub_context_new();
+		sub_var_set(scp, "Name", "%S", args->string[0]);
+		error_with_position
 		(
-			"%s: requires one or more arguments",
-			args->wl_word[0]->str_text
+			pp,
+			scp,
+			i18n("$name: requires one or more arguments")
 		);
+		sub_context_delete(scp);
 		return -1;
 	}
-	for (j = 1; j < args->wl_nwords; j++)
+	for (j = 1; j < args->nstrings; j++)
 	{
 		string_ty	*s;
 
-		s = os_pathname(args->wl_word[j]);
+		s = os_pathname(args->string[j]);
 		if (!s)
 			return -1;
-		wl_append(result, s);
+		string_list_append(result, s);
 		str_free(s);
 	}
 	return 0;
