@@ -1,6 +1,6 @@
 /*
  *	cook - file construction tool
- *	Copyright (C) 1994 Peter Miller.
+ *	Copyright (C) 1994, 1997, 1998 Peter Miller;
  *	All rights reserved.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -15,40 +15,52 @@
  *
  *	You should have received a copy of the GNU General Public License
  *	along with this program; if not, write to the Free Software
- *	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
  *
  * MANIFEST: functions to implement the split builtin function
  */
 
 #include <builtin/split.h>
-#include <expr.h>
+#include <error_intl.h>
+#include <expr/position.h>
+#include <str_list.h>
+#include <trace.h>
 
 
 int
-builtin_split(result, arg)
-	wlist		*result;
-	wlist		*arg;
+builtin_split(result, arg, pp, ocp)
+	string_list_ty	*result;
+	const string_list_ty *arg;
+	const expr_position_ty *pp;
+	const struct opcode_context_ty *ocp;
 {
 	size_t		j;
 
-	if (arg->wl_nwords < 2)
+	trace(("split\n"));
+	if (arg->nstrings < 2)
 	{
-		expr_error
+		sub_context_ty	*scp;
+
+		scp = sub_context_new();
+		sub_var_set(scp, "Name", "%S", arg->string[0]);
+		error_with_position
 		(
-			"%s: requires one or more arguments",
-			arg->wl_word[0]->str_text
+			pp,
+			scp,
+			i18n("$name: requires one or more arguments")
 		);
+		sub_context_delete(scp);
 		return -1;
 	}
-	for (j = 2; j < arg->wl_nwords; ++j)
+	for (j = 2; j < arg->nstrings; ++j)
 	{
-		wlist		wl;
+		string_list_ty	wl;
 		size_t		k;
 
-		str2wl(&wl, arg->wl_word[j], arg->wl_word[1]->str_text, 0);
-		for (k = 0; k < wl.wl_nwords; ++k)
-			wl_append(result, wl.wl_word[k]);
-		wl_free(&wl);
+		str2wl(&wl, arg->string[j], arg->string[1]->str_text, 0);
+		for (k = 0; k < wl.nstrings; ++k)
+			string_list_append(result, wl.string[k]);
+		string_list_destructor(&wl);
 	}
 	return 0;
 }
