@@ -1,6 +1,6 @@
 /*
  *	cook - file construction tool
- *	Copyright (C) 1994 Peter Miller.
+ *	Copyright (C) 1994, 1997, 1998 Peter Miller;
  *	All rights reserved.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -15,42 +15,54 @@
  *
  *	You should have received a copy of the GNU General Public License
  *	along with this program; if not, write to the Free Software
- *	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
  *
  * MANIFEST: functions to implement builtin subst function
  */
 
 #include <builtin/subst.h>
-#include <expr.h>
+#include <error_intl.h>
+#include <expr/position.h>
+#include <str_list.h>
+#include <trace.h>
 
 
 int
-builtin_subst(result, arg)
-	wlist		*result;
-	wlist		*arg;
+builtin_subst(result, arg, pp, ocp)
+	string_list_ty	*result;
+	const string_list_ty *arg;
+	const expr_position_ty *pp;
+	const struct opcode_context_ty *ocp;
 {
 	size_t		j;
 	string_ty	*s;
 
-	if (arg->wl_nwords < 3)
+	trace(("subst\n"));
+	if (arg->nstrings < 3)
 	{
-		expr_error
+		sub_context_ty	*scp;
+
+		scp = sub_context_new();
+		sub_var_set(scp, "Name", "%S", arg->string[0]);
+		error_with_position
 		(
-			"%s: requires at least two arguments",
-			arg->wl_word[0]->str_text
+			pp,
+			scp,
+			i18n("$name: requires two or more arguments")
 		);
+		sub_context_delete(scp);
 		return -1;
 	}
-	for (j = 3; j < arg->wl_nwords; ++j)
+	for (j = 3; j < arg->nstrings; ++j)
 	{
 		s =
 			str_substitute
 			(
-				arg->wl_word[1],
-				arg->wl_word[2],
-				arg->wl_word[j]
+				arg->string[1],
+				arg->string[2],
+				arg->string[j]
 			);
-		wl_append(result, s);
+		string_list_append(result, s);
 		str_free(s);
 	}
 	return 0;
