@@ -1,6 +1,6 @@
 /*
  *	cook - file construction tool
- *	Copyright (C) 1994 Peter Miller.
+ *	Copyright (C) 1994, 1997 Peter Miller;
  *	All rights reserved.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -15,7 +15,7 @@
  *
  *	You should have received a copy of the GNU General Public License
  *	along with this program; if not, write to the Free Software
- *	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
  *
  * MANIFEST: functions to implement the builtin suffix function
  */
@@ -23,31 +23,36 @@
 #include <ac/string.h>
 
 #include <builtin/suffix.h>
+#include <str_list.h>
+#include <trace.h>
 
 
 int
-builtin_suffix(result, arg)
-	wlist		*result;
-	wlist		*arg;
+builtin_suffix(result, arg, pp, ocp)
+	string_list_ty	*result;
+	const string_list_ty *arg;
+	const struct expr_position_ty *pp;
+	const struct opcode_context_ty *ocp;
 {
 	size_t		j;
 	string_ty	*s;
 	char		*cp;
 	char		*ep;
 
-	for (j = 1; j < arg->wl_nwords; ++j)
+	trace(("suffix\n"));
+	for (j = 1; j < arg->nstrings; ++j)
 	{
-		s = arg->wl_word[j];
+		s = arg->string[j];
 		cp = strrchr(s->str_text, '/');
 		if (!cp)
 			cp = s->str_text;
 		ep = strrchr(cp, '.');
 		if (!ep || ep == cp)
-			wl_append(result, str_false);
+			string_list_append(result, str_false);
 		else
 		{
 			s = str_from_c(ep);
-			wl_append(result, s);
+			string_list_append(result, s);
 			str_free(s);
 		}
 	}
