@@ -1,6 +1,6 @@
 /*
  *	cook - file construction tool
- *	Copyright (C) 1991, 1992, 1993, 1994 Peter Miller.
+ *	Copyright (C) 1991, 1992, 1993, 1994, 1997, 1998 Peter Miller;
  *	All rights reserved.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -15,7 +15,7 @@
  *
  *	You should have received a copy of the GNU General Public License
  *	along with this program; if not, write to the Free Software
- *	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
  *
  * MANIFEST: functions to implement the builtin text functions
  *
@@ -30,9 +30,12 @@
 #include <ac/string.h>
 
 #include <builtin/text.h>
-#include <error.h>
-#include <expr.h>
+#include <error_intl.h>
+#include <expr/position.h>
 #include <mem.h>
+#include <quote.h>
+#include <str_list.h>
+#include <trace.h>
 
 
 /*
@@ -40,7 +43,7 @@
  *	builtin_upcase - upcase strings
  *
  * SYNOPSIS
- *	int builtin_upcase(wlist *result, wlist *args);
+ *	int builtin_upcase(string_list_ty *result, string_list_ty *args);
  *
  * DESCRIPTION
  *	Defined is a built-in function of cook, described as follows:
@@ -53,34 +56,43 @@
  * CAVEAT
  *	The returned result is in dynamic memory.
  *	It is the responsibility of the caller to dispose of
- *	the result when it is finished, with a wl_free() call.
+ *	the result when it is finished, with a string_list_destructor() call.
  */
 
 int
-builtin_upcase(result, args)
-	wlist		*result;
-	wlist		*args;
+builtin_upcase(result, args, pp, ocp)
+	string_list_ty	*result;
+	const string_list_ty *args;
+	const expr_position_ty *pp;
+	const struct opcode_context_ty *ocp;
 {
 	int		j;
 
+	trace(("upcase\n"));
 	assert(result);
 	assert(args);
-	assert(args->wl_nwords);
-	if (args->wl_nwords < 2)
+	assert(args->nstrings);
+	if (args->nstrings < 2)
 	{
-		expr_error
+		sub_context_ty	*scp;
+
+		scp = sub_context_new();
+		sub_var_set(scp, "Name", "%S", args->string[0]);
+		error_with_position
 		(
-			"%s: requires one or more arguments",
-			args->wl_word[0]->str_text
+			pp,
+			scp,
+			i18n("$name: requires one or more arguments")
 		);
+		sub_context_delete(scp);
 		return -1;
 	}
-	for (j = 1; j < args->wl_nwords; j++)
+	for (j = 1; j < args->nstrings; j++)
 	{
 		string_ty *s;
 
-		s = str_upcase(args->wl_word[j]);
-		wl_append(result, s);
+		s = str_upcase(args->string[j]);
+		string_list_append(result, s);
 		str_free(s);
 	}
 	return 0;
@@ -92,7 +104,7 @@ builtin_upcase(result, args)
  *	builtin_downcase - downcase strings
  *
  * SYNOPSIS
- *	int builtin_downcase(wlist *result, wlist *args);
+ *	int builtin_downcase(string_list_ty *result, string_list_ty *args);
  *
  * DESCRIPTION
  *	Defined is a built-in function of cook, described as follows:
@@ -105,34 +117,43 @@ builtin_upcase(result, args)
  * CAVEAT
  *	The returned result is in dynamic memory.
  *	It is the responsibility of the caller to dispose of
- *	the result when it is finished, with a wl_free() call.
+ *	the result when it is finished, with a string_list_destructor() call.
  */
 
 int
-builtin_downcase(result, args)
-	wlist		*result;
-	wlist		*args;
+builtin_downcase(result, args, pp, ocp)
+	string_list_ty	*result;
+	const string_list_ty *args;
+	const expr_position_ty *pp;
+	const struct opcode_context_ty *ocp;
 {
 	int		j;
 
+	trace(("downcase\n"));
 	assert(result);
 	assert(args);
-	assert(args->wl_nwords);
-	if (args->wl_nwords < 2)
+	assert(args->nstrings);
+	if (args->nstrings < 2)
 	{
-		expr_error
+		sub_context_ty	*scp;
+
+		scp = sub_context_new();
+		sub_var_set(scp, "Name", "%S", args->string[0]);
+		error_with_position
 		(
-			"%s: requires one or more arguments",
-			args->wl_word[0]->str_text
+			pp,
+			scp,
+			i18n("$name: requires one or more arguments")
 		);
+		sub_context_delete(scp);
 		return -1;
 	}
-	for (j = 1; j < args->wl_nwords; j++)
+	for (j = 1; j < args->nstrings; j++)
 	{
 		string_ty	*s;
 
-		s = str_downcase(args->wl_word[j]);
-		wl_append(result, s);
+		s = str_downcase(args->string[j]);
+		string_list_append(result, s);
 		str_free(s);
 	}
 	return 0;
@@ -144,7 +165,7 @@ builtin_downcase(result, args)
  *	builtin_prepost - add prefix and suffix
  *
  * SYNOPSIS
- *	int builtin_prepost(wlist *result, wlist *args);
+ *	int builtin_prepost(string_list_ty *result, string_list_ty *args);
  *
  * DESCRIPTION
  *	Prepost is a built-in function of cook, described as follows:
@@ -158,40 +179,49 @@ builtin_downcase(result, args)
  * CAVEAT
  *	The returned result is in dynamic memory.
  *	It is the responsibility of the caller to dispose of
- *	the result when it is finished, with a wl_free() call.
+ *	the result when it is finished, with a string_list_destructor() call.
  */
 
 int
-builtin_prepost(result, args)
-	wlist		*result;
-	wlist		*args;
+builtin_prepost(result, args, pp, ocp)
+	string_list_ty	*result;
+	const string_list_ty *args;
+	const expr_position_ty *pp;
+	const struct opcode_context_ty *ocp;
 {
 	int		j;
 
+	trace(("prepost\n"));
 	assert(result);
 	assert(args);
-	assert(args->wl_nwords);
-	if (args->wl_nwords < 3)
+	assert(args->nstrings);
+	if (args->nstrings < 3)
 	{
-		expr_error
+		sub_context_ty	*scp;
+
+		scp = sub_context_new();
+		sub_var_set(scp, "Name", "%S", args->string[0]);
+		error_with_position
 		(
-			"%s: requires at least two arguments",
-			args->wl_word[0]->str_text
+			pp,
+			scp,
+			i18n("$name: requires two or more arguments")
 		);
+		sub_context_delete(scp);
 		return -1;
 	}
-	for (j = 3; j < args->wl_nwords; j++)
+	for (j = 3; j < args->nstrings; j++)
 	{
 		string_ty	*s;
 
 		s =
 			str_cat_three
 			(
-				args->wl_word[1],
-				args->wl_word[j],
-				args->wl_word[2]
+				args->string[1],
+				args->string[j],
+				args->string[2]
 			);
-		wl_append(result, s);
+		string_list_append(result, s);
 		str_free(s);
 	}
 	return 0;
@@ -203,7 +233,7 @@ builtin_prepost(result, args)
  *	builtin_head - head of a wordlist
  *
  * SYNOPSIS
- *	int builtin_fromto(wlist *result, wlist *args);
+ *	int builtin_fromto(string_list_ty *result, string_list_ty *args);
  *
  * DESCRIPTION
  *	Head is a built-in function of cook, described as follows:
@@ -216,19 +246,22 @@ builtin_prepost(result, args)
  * CAVEAT
  *	The returned result is in dynamic memory.
  *	It is the responsibility of the caller to dispose of
- *	the result when it is finished, with a wl_free() call.
+ *	the result when it is finished, with a string_list_destructor() call.
  */
 
 int
-builtin_head(result, args)
-	wlist		*result;
-	wlist		*args;
+builtin_head(result, args, pp, ocp)
+	string_list_ty	*result;
+	const string_list_ty *args;
+	const expr_position_ty *pp;
+	const struct opcode_context_ty *ocp;
 {
+	trace(("head\n"));
 	assert(result);
 	assert(args);
-	assert(args->wl_nwords);
-	if (args->wl_nwords >= 2)
-		wl_append(result, args->wl_word[1]);
+	assert(args->nstrings);
+	if (args->nstrings >= 2)
+		string_list_append(result, args->string[1]);
 	return 0;
 }
 
@@ -238,7 +271,7 @@ builtin_head(result, args)
  *	builtin_tail - tail of a wordlist
  *
  * SYNOPSIS
- *	int builtin_tail(wlist *result, wlist *args);
+ *	int builtin_tail(string_list_ty *result, string_list_ty *args);
  *
  * DESCRIPTION
  *	Tail is a built-in function of cook, described as follows:
@@ -252,21 +285,24 @@ builtin_head(result, args)
  * CAVEAT
  *	The returned result is in dynamic memory.
  *	It is the responsibility of the caller to dispose of
- *	the result when it is finished, with a wl_free() call.
+ *	the result when it is finished, with a string_list_destructor() call.
  */
 
 int
-builtin_tail(result, args)
-	wlist		*result;
-	wlist		*args;
+builtin_tail(result, args, pp, ocp)
+	string_list_ty	*result;
+	const string_list_ty *args;
+	const expr_position_ty *pp;
+	const struct opcode_context_ty *ocp;
 {
 	int		j;
 
+	trace(("tail\n"));
 	assert(result);
 	assert(args);
-	assert(args->wl_nwords);
-	for (j = 2; j < args->wl_nwords; j++)
-		wl_append(result, args->wl_word[j]);
+	assert(args->nstrings);
+	for (j = 2; j < args->nstrings; j++)
+		string_list_append(result, args->string[j]);
 	return 0;
 }
 
@@ -276,7 +312,7 @@ builtin_tail(result, args)
  *	builtin_catenate - catenate a wordlist
  *
  * SYNOPSIS
- *	int builtin_catenate(wlist *result, wlist *args);
+ *	int builtin_catenate(string_list_ty *result, string_list_ty *args);
  *
  * DESCRIPTION
  *	Catenate is a built-in function of cook, described as follows:
@@ -289,13 +325,15 @@ builtin_tail(result, args)
  * CAVEAT
  *	The returned result is in dynamic memory.
  *	It is the responsibility of the caller to dispose of
- *	the result when it is finished, with a wl_free() call.
+ *	the result when it is finished, with a string_list_destructor() call.
  */
 
 int
-builtin_catenate(result, args)
-	wlist		*result;
-	wlist		*args;
+builtin_catenate(result, args, pp, ocp)
+	string_list_ty	*result;
+	const string_list_ty *args;
+	const expr_position_ty *pp;
+	const struct opcode_context_ty *ocp;
 {
 	int		j;
 	static char	*tmp;
@@ -304,20 +342,21 @@ builtin_catenate(result, args)
 	char		*pos;
 	string_ty	*s;
 
+	trace(("catenate\n"));
 	assert(result);
 	assert(args);
-	assert(args->wl_nwords);
-	if (args->wl_nwords < 2)
+	assert(args->nstrings);
+	if (args->nstrings < 2)
 		return 0;
-	if (args->wl_nwords == 2)
+	if (args->nstrings == 2)
 	{
-		wl_append(result, args->wl_word[1]);
+		string_list_append(result, args->string[1]);
 		return 0;
 	}
 
 	length = 0;
-	for (j = 1; j < args->wl_nwords; j++)
-		length += args->wl_word[j]->str_length;
+	for (j = 1; j < args->nstrings; j++)
+		length += args->string[j]->str_length;
 	if (!tmp)
 	{
 		tmplen = length;
@@ -334,14 +373,14 @@ builtin_catenate(result, args)
 		}
 	}
 	pos = tmp;
-	for (j = 1; j < args->wl_nwords; j++)
+	for (j = 1; j < args->nstrings; j++)
 	{
-		s = args->wl_word[j];
+		s = args->string[j];
 		memcpy(pos, s->str_text, s->str_length);
 		pos += s->str_length;
 	}
 	s = str_n_from_c(tmp, length);
-	wl_append(result, s);
+	string_list_append(result, s);
 	str_free(s);
 	return 0;
 }
@@ -352,7 +391,7 @@ builtin_catenate(result, args)
  *	builtin_count - length of a word list
  *
  * SYNOPSIS
- *	int builtin_count(wlist *result, wlist *args);
+ *	int builtin_count(string_list_ty *result, string_list_ty *args);
  *
  * DESCRIPTION
  *	Count is a built-in function of cook, described as follows:
@@ -365,21 +404,24 @@ builtin_catenate(result, args)
  * CAVEAT
  *	The returned result is in dynamic memory.
  *	It is the responsibility of the caller to dispose of
- *	the result when it is finished, with a wl_free() call.
+ *	the result when it is finished, with a string_list_destructor() call.
  */
 
 int
-builtin_count(result, args)
-	wlist		*result;
-	wlist		*args;
+builtin_count(result, args, pp, ocp)
+	string_list_ty	*result;
+	const string_list_ty *args;
+	const expr_position_ty *pp;
+	const struct opcode_context_ty *ocp;
 {
 	string_ty	*s;
 
+	trace(("count\n"));
 	assert(result);
 	assert(args);
-	assert(args->wl_nwords);
-	s = str_format("%ld", args->wl_nwords - 1);
-	wl_append(result, s);
+	assert(args->nstrings);
+	s = str_format("%ld", args->nstrings - 1);
+	string_list_append(result, s);
 	str_free(s);
 	return 0;
 }
@@ -390,7 +432,7 @@ builtin_count(result, args)
  *	builtin_quote - quote the arguments
  *
  * SYNOPSIS
- *	int builtin_quote(wlist *result, wlist *args);
+ *	int builtin_quote(string_list_ty *result, string_list_ty *args);
  *
  * DESCRIPTION
  *	The quote function is a built-in of cook, described as follows:
@@ -403,81 +445,27 @@ builtin_count(result, args)
  * CAVEAT
  *	The returned result is in dynamic memory.
  *	It is the responsibility of the caller to dispose of
- *	the result when it is finished, with a wl_free() call.
+ *	the result when it is finished, with a string_list_destructor() call.
  */
 
 int
-builtin_quote(result, args)
-	wlist		*result;
-	wlist		*args;
+builtin_quote(result, args, pp, ocp)
+	string_list_ty	*result;
+	const string_list_ty *args;
+	const expr_position_ty *pp;
+	const struct opcode_context_ty *ocp;
 {
-	int		j;
-	static char	*tmp;
-	static size_t	tmp_len;
-	size_t		len;
-	char		*cp1;
-	char		*cp2;
-	char		*cp3;
+	size_t		j;
 	string_ty	*s;
-	static char	special[] = "\bb\ff\nn\rr\tt";
 
+	trace(("quote\n"));
 	assert(result);
 	assert(args);
-	assert(args->wl_nwords);
-	for (j = 1; j < args->wl_nwords; ++j)
+	assert(args->nstrings);
+	for (j = 1; j < args->nstrings; ++j)
 	{
-		len = 2;
-		for (cp1 = args->wl_word[j]->str_text; *cp1; ++cp1)
-		{
-			if (*cp1 < ' ' || *cp1 > '~')
-			{
-				if (strchr(special, *cp1))
-					len += 2;
-				else
-					len += 4;
-			}
-			else
-			{
-				if (strchr("\"\\", *cp1))
-					++len;
-				++len;
-			}
-		}
-		if (len > tmp_len)
-		{
-			tmp_len = len;
-			tmp = mem_change_size(tmp, tmp_len);
-		}
-		cp2 = tmp;
-		*cp2++ = '"';
-		for (cp1 = args->wl_word[j]->str_text; *cp1; ++cp1)
-		{
-			if (*cp1 < ' ' || *cp1 > '~')
-			{
-				cp3 = strchr(special, *cp1);
-				if (cp3)
-				{
-					*cp2++ = '\\';
-					*cp2++= cp3[1];
-				}
-				else
-				{
-					*cp2++ = '\\';
-					*cp2++ = '0' + ((*cp1 >> 6) & 3);
-					*cp2++ = '0' + ((*cp1 >> 3) & 7);
-					*cp2++ = '0' + (*cp1 & 7);
-				}
-			}
-			else
-			{
-				if (strchr("\"\\", *cp1))
-					*cp2++ = '\\';
-				*cp2++ = *cp1;
-			}
-		}
-		*cp2 = '"';
-		s = str_n_from_c(tmp, len);
-		wl_append(result, s);
+		s = quoted(args->string[j]);
+		string_list_append(result, s);
 		str_free(s);
 	}
 	return 0;
@@ -489,7 +477,7 @@ builtin_quote(result, args)
  *	builtin_sort - sort the arguments
  *
  * SYNOPSIS
- *	int builtin_sort(wlist *result, wlist *args);
+ *	int builtin_sort(string_list_ty *result, string_list_ty *args);
  *
  * DESCRIPTION
  *	The builtin_sort function is a built-in of cook, described as follows:
@@ -502,7 +490,7 @@ builtin_quote(result, args)
  * CAVEAT
  *	The returned result is in dynamic memory.
  *	It is the responsibility of the caller to dispose of
- *	the result when it is finished, with a wl_free() call.
+ *	the result when it is finished, with a string_list_destructor() call.
  */
 
 static int cmp _((const void *, const void *));
@@ -522,16 +510,19 @@ cmp(va, vb)
 
 
 int
-builtin_sort(result, args)
-	wlist		*result;
-	wlist		*args;
+builtin_sort(result, args, pp, ocp)
+	string_list_ty	*result;
+	const string_list_ty *args;
+	const expr_position_ty *pp;
+	const struct opcode_context_ty *ocp;
 {
 	int		j;
 	int		start;
 
+	trace(("sort\n"));
 	assert(result);
 	assert(args);
-	switch (args->wl_nwords)
+	switch (args->nstrings)
 	{
 	case 0:
 		assert(0);
@@ -540,17 +531,17 @@ builtin_sort(result, args)
 		return 0;
 
 	case 2:
-		wl_append(result, args->wl_word[1]);
+		string_list_append(result, args->string[1]);
 		return 0;
 	}
-	start = result->wl_nwords;
-	for (j = 1; j < args->wl_nwords; ++j)
-		wl_append(result, args->wl_word[j]);
+	start = result->nstrings;
+	for (j = 1; j < args->nstrings; ++j)
+		string_list_append(result, args->string[j]);
 	qsort
 	(
-		&result->wl_word[start],
-		args->wl_nwords - 1,
-		sizeof(result->wl_word[0]),
+		&result->string[start],
+		args->nstrings - 1,
+		sizeof(result->string[0]),
 		cmp
 	);
 	return 0;
