@@ -1,6 +1,6 @@
 /*
  *	cook - file construction tool
- *	Copyright (C) 1994 Peter Miller.
+ *	Copyright (C) 1994, 1997 Peter Miller;
  *	All rights reserved.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -15,7 +15,7 @@
  *
  *	You should have received a copy of the GNU General Public License
  *	along with this program; if not, write to the Free Software
- *	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
  *
  * MANIFEST: interface definition for cook/fngrprnt_lex.c
  */
@@ -28,6 +28,6 @@
 void fngrprnt_lex_open _((char *));
 void fngrprnt_lex_close _((void));
 int fngrprnt_lex _((void));
-void fngrprnt_error _((char *, ...));
+void fngrprnt_error _((char *)); /* i18n keyword */
 
 #endif /* COOK_FNGRPRNT_LEX_H */
