@@ -1,6 +1,6 @@
 /*
  *	cook - file construction tool
- *	Copyright (C) 1990, 1991, 1992, 1993, 1994 Peter Miller.
+ *	Copyright (C) 1990, 1991, 1992, 1993, 1994, 1997 Peter Miller;
  *	All rights reserved.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -15,28 +15,42 @@
  *
  *	You should have received a copy of the GNU General Public License
  *	along with this program; if not, write to the Free Software
- *	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
  *
  * MANIFEST: interface definition for cook/id.c
  */
 
-#ifndef ID_H
-#define ID_H
+#ifndef COOK_ID_H
+#define COOK_ID_H
 
-#include <word.h>
+#include <str_list.h>
+
+struct expr_position_ty; /* existence */
+struct opcode_context_ty; /* existence */
 
 extern string_ty *id_need;
 extern string_ty *id_younger;
-extern string_ty *id_friend;
+extern string_ty *id_targets;
 extern string_ty *id_target;
 extern string_ty *id_search_list;
 
+typedef struct id_ty id_ty;
+struct id_ty
+{
+	struct id_method_ty *method;
+};
+
 void id_initialize _((void));
 void id_reset _((void));
-int id_search _((string_ty *, wlist *));
-void id_assign _((string_ty *, wlist *));
-void id_assign_push _((string_ty *, wlist *));
-void id_delete _((string_ty *));
+id_ty *id_search _((string_ty *));
+id_ty *id_search_fuzzy _((string_ty *, string_ty **));
+void id_assign _((string_ty *, id_ty *));
+void id_assign_push _((string_ty *, id_ty *));
+#define id_delete /-/-/-/
+void id_unassign _((string_ty *));
 void id_dump _((char *));
 
-#endif /* ID_H */
+int id_interpret _((id_ty *, struct opcode_context_ty *,
+	const struct expr_position_ty *));
+
+#endif /* COOK_ID_H */
