@@ -1,6 +1,6 @@
 /*
  *	cook - file construction tool
- *	Copyright (C) 1990, 1991, 1992, 1993, 1994 Peter Miller.
+ *	Copyright (C) 1990, 1991, 1992, 1993, 1994, 1995, 1997, 1998 Peter Miller;
  *	All rights reserved.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -15,7 +15,7 @@
  *
  *	You should have received a copy of the GNU General Public License
  *	along with this program; if not, write to the Free Software
- *	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
  *
  * MANIFEST: functions to perform lexical analysis on cookbooks
  *
@@ -25,23 +25,29 @@
  * space.
  */
 
-#include <ctype.h>
+#include <ac/ctype.h>
 #include <ac/stddef.h>
 #include <ac/string.h>
 #include <ac/stdlib.h>
-#include <stdio.h>
+#include <ac/stdarg.h>
+#include <ac/stdio.h>
 
 #include <error.h>
+#include <error_intl.h>
 #include <expr.h>
+#include <expr/list.h> /* for parse.gen.h */
 #include <hashline.h>
 #include <lex.h>
+#include <lex/filename.h>
+#include <lex/filenamelist.h>
 #include <mem.h>
 #include <option.h>
-#include <s-v-arg.h>
 #include <stmt.h>
+#include <stmt/list.h> /* for parse.gen.h */
 #include <symtab.h>
+#include <star.h>
+#include <str_list.h>
 #include <trace.h>
-#include <word.h>
 #include <parse.gen.h>	/* must be last */
 #include <hashline.gen.h> /* must be last */
 
@@ -75,13 +81,14 @@ struct meta_ty
 typedef struct lex_ty lex_ty;
 struct lex_ty
 {
-	string_ty	*l_name;	/* name of file being analysed	*/
+	lex_filename_ty	filename;	/* name of file being analysed	*/
 	FILE		*l_file;	/* open file structure of the file */
-	int		l_line;		/* the line number we are up to	*/
+	long		l_line;		/* the line number we are up to	*/
 	meta_ty		l_mback;	/* backup for meta chars	*/
 	lex_ty		*l_chain;	/* file this one is an insert from */
 	short		l_bol;
-	wlist		pending_includes;
+	lex_filename_list_ty pending_include_list;
+	int		have_said_eof_token;
 };
 
 typedef struct stracc stracc;
@@ -102,6 +109,7 @@ static	int		passing;
 static	int		catted;
 static	symtab_ty	*parse_symtab;
 static	symtab_ty	*hash_symtab;
+static	symtab_ty	*hash_directive_symtab;
 
 
 /*
@@ -132,15 +140,20 @@ lex_initialize()
 	static keyword_ty parse_keyword[] =
 	{
 		{ ":",			COLON,			},
+		{ "::",			COLON2,			},
 		{ "=",			EQUALS,			},
 		{ "[",			LBRAK,			},
 		{ "]",			RBRAK,			},
 		{ "data",		DATA,			},
 		{ "else",		ELSE,			},
 		{ "fail",		FAIL,			},
+		{ "function",		FUNCTION,		},
+		{ "host-binding",	HOST_BINDING,		},
 		{ "if",			IF,			},
 		{ "loop",		LOOP,			},
+		{ "return",		RETURN,			},
 		{ "set",		SET,			},
+		{ "single-thread",	SINGLE_THREAD,		},
 		{ "then",		THEN,			},
 		{ "{"/*}*/,		LBRACE,			},
 		{ /*{*/"}",		RBRACE,			},
@@ -148,10 +161,8 @@ lex_initialize()
 		{ "loopstop",		LOOPSTOP,		},
 		{ "unsetenv",		UNSETENV,		},
 	};
-	static keyword_ty hash_keyword[] =
+	static keyword_ty hash_directive[] =
 	{
-		{ "[",			LBRAK,			},
-		{ "]",			RBRAK,			},
 		{ "elif",		HASH_ELIF,		},
 		{ "else",		HASH_ELSE,		},
 		{ "endif",		HASH_ENDIF,		},
@@ -161,8 +172,14 @@ lex_initialize()
 		{ "include",		HASH_INCLUDE,		},
 		{ "include-cooked",	HASH_INCLUDE_COOKED,	},
 		{ "include-cooked-nowarn", HASH_INCLUDE_COOKED2, },
+		{ "line",		HASH_LINE,		},
 		{ "pragma",		HASH_PRAGMA,		},
 	};
+	static keyword_ty hash_keyword[] =
+	{
+		{ "[",			LBRAK,			},
+		{ "]",			RBRAK,			},
+	};
 	keyword_ty	*kp;
 
 	trace(("init_reserved()\n{\n"/*}*/));
@@ -189,6 +206,17 @@ lex_initialize()
 		symtab_assign(hash_symtab, s, &kp->k_token);
 		str_free(s);
 	}
+
+	if (!hash_directive_symtab)
+		hash_directive_symtab = symtab_alloc(SIZEOF(hash_directive));
+	for (kp = hash_directive; kp < ENDOF(hash_directive); ++kp)
+	{
+		string_ty	*s;
+
+		s = str_from_c(kp->k_name);
+		symtab_assign(hash_directive_symtab, s, &kp->k_token);
+		str_free(s);
+	}
 	trace((/*{*/"}\n"));
 }
 
@@ -209,43 +237,65 @@ lex_initialize()
  */
 
 void
-lex_open(filename)
-	string_ty	*filename;
+lex_open(logical, physical)
+	string_ty	*logical;
+	string_ty	*physical;
 {
 	lex_ty		*new;
 
-	trace(("lex_open(filename = %08lX)\n{\n"/*}*/, filename));
-	trace_string(filename->str_text);
+	trace(("lex_open(filename = %08lX)\n{\n"/*}*/, logical));
+	trace_string(logical->str_text);
+	if (!physical)
+		physical = logical;
 	if (!root)
 		hashline_reset();
 	for (new = root; new; new = new->l_chain)
 	{
-		if (str_equal(filename, new->l_name))
+		if (str_equal(physical, new->filename.physical))
 		{
-			lex_error("%S: recursive include", filename);
+			sub_context_ty	*scp;
+
+			scp = sub_context_new();
+			sub_var_set(scp, "File_Name", "%S", physical);
+			lex_error(scp, i18n("$filename: recursive include"));
+			sub_context_delete(scp);
 			return;
 		}
 	}
 	new = mem_alloc_clear(sizeof(lex_ty));
-	new->l_name = str_copy(filename);
-	new->l_file = fopen(filename->str_text, "r");
+	lex_filename_constructor(&new->filename, logical, physical);
+	new->l_file = fopen(physical->str_text, "r");
 	if (!new->l_file)
-		nfatal("cookbook \"%S\"", filename);
+	{
+		sub_context_ty	*scp;
+
+		scp = sub_context_new();
+		sub_errno_set(scp);
+		sub_var_set(scp, "File_Name", "%S", physical);
+		fatal_intl(scp, i18n("open $filename: $errno"));
+		/* NOTREACHED */
+	}
 	new->l_line = 1;
 	new->l_bol = 1;
 	new->l_chain = root;
-	wl_zero(&new->pending_includes);
+	lex_filename_list_constructor(&new->pending_include_list);
+	new->have_said_eof_token = 0;
 	root = new;
 	trace((/*{*/"}\n"));
 }
 
 
 void
-lex_open_include(s)
-	string_ty	*s;
+lex_open_include(logical, physical)
+	string_ty	*logical;
+	string_ty	*physical;
 {
 	assert(root);
-	wl_append(&root->pending_includes, s);
+	lex_filename_list_push_back
+	(
+		&root->pending_include_list,
+		lex_filename_new(logical, physical)
+	);
 }
 
 
@@ -274,9 +324,18 @@ lex_close()
 	fclose(root->l_file);
 	root = old->l_chain;
 	if (!root && errcnt)
-		exit(1);
-	str_free(old->l_name);
-	wl_free(&old->pending_includes);
+	{
+		sub_context_ty	*scp;
+
+		scp = sub_context_new();
+		sub_var_set(scp, "File_Name", "%S", old->filename.logical);
+		sub_var_set(scp, "Number", "%d", errcnt);
+		sub_var_optional(scp, "Number");
+		fatal_intl(scp, i18n("$filename: found $number fatal errors"));
+		/* NOTREACHED */
+	}
+	lex_filename_destructor(&old->filename);
+	lex_filename_list_destructor(&old->pending_include_list);
 	free(old);
 	trace((/*{*/"}\n"));
 }
@@ -302,7 +361,7 @@ static void illeof _((void));
 static void
 illeof()
 {
-	lex_error("unexpected end of file");
+	lex_error(0, i18n("unexpected end of file"));
 	exit(1);
 }
 
@@ -337,15 +396,18 @@ byte()
 		 * leap into include file if possible
 		 */
 		assert(root);
-		if (root->pending_includes.wl_nwords)
+		if (root->pending_include_list.length)
 		{
-			string_ty	*fn;
+			lex_filename_ty	*fnp;
 
-			fn = str_copy(root->pending_includes.wl_word[0]);
-			wl_delete(&root->pending_includes, fn);
-			lex_open(fn);
-			str_free(fn);
-			star();
+			fnp =
+				lex_filename_list_pop_front
+				(
+					&root->pending_include_list
+				);
+			lex_open(fnp->logical, fnp->physical);
+			lex_filename_delete(fnp);
+			star_as_specified('+');
 		}
 
 		/*
@@ -355,14 +417,33 @@ byte()
 		if (c != EOF)
 			break;
 		if (ferror(root->l_file))
-			nfatal("read %S", root->l_name);
-		if (!root->l_chain)
+		{
+			sub_context_ty	*scp;
+
+			scp = sub_context_new();
+			sub_errno_set(scp);
+			sub_var_set
+			(
+				scp,
+				"File_Name",
+				"%S",
+				root->filename.physical
+			);
+			fatal_intl(scp, i18n("read $filename: $errno"));
+			/* NOTREACHED */
+		}
+		if (!root->l_chain || !root->have_said_eof_token)
 			break;
 		lex_close();
 	}
 	if (!isprint(c) && c != EOF && !strchr("\t\n\f", c))
 	{
-		lex_error("illegal '\\%o' character", (unsigned char)c);
+		sub_context_ty	*scp;
+
+		scp = sub_context_new();
+		sub_var_set(scp, "Name", "\\%o", (unsigned char)c);
+		lex_error(scp, i18n("illegal '$name' character"));
+		sub_context_delete(scp);
 		c = ' ';
 	}
 	if (mode != LM_DATA && root->l_bol && c == '#')
@@ -938,7 +1019,7 @@ meta_undo(val)
  *	lex_error - print a file location specific error message
  *
  * SYNOPSIS
- *	void lex_error(char *s, ...);
+ *	void lex_error(char *s);
  *
  * DESCRIPTION
  *	Lex_error prints a file location specific error message.
@@ -946,22 +1027,47 @@ meta_undo(val)
  */
 
 void
-lex_error(s sva_last)
+lex_error(scp, s)
+	sub_context_ty	*scp;
 	char		*s;
-	sva_last_decl
 {
 	string_ty	*buffer;
-	va_list		ap;
+	int		need_to_delete;
+
+	if (scp)
+		need_to_delete = 0;
+	else
+	{
+		scp = sub_context_new();
+		need_to_delete = 1;
+	}
 
-	sva_init(ap, s);
 	assert(root);
-	buffer = str_vformat(s, ap);
-	va_end(ap);
-	error("%S: %d: %S", root->l_name, root->l_line, buffer);
+	buffer = subst_intl(scp, s);
+
+	/* re-use the substitution context */
+	sub_var_set(scp, "File_Name", "%S", root->filename.logical);
+	sub_var_set(scp, "Number", "%ld", root->l_line);
+	sub_var_set(scp, "MeSsaGe", "%S", buffer);
+	error_intl(scp, i18n("$filename: $number: $message"));
 	str_free(buffer);
 	if (++errcnt > 20)
-		fatal("%S: too many errors", root->l_name);
+	{
+		/* re-use the substitution context */
+		sub_var_set(scp, "File_Name", "%S", root->filename.logical);
+		fatal_intl(scp, i18n("$filename: too many fatal errors"));
+	}
 	option_set_errors();
+
+	if (need_to_delete)
+		sub_context_delete(scp);
+}
+
+void
+parse_error(s)
+	char		*s;
+{
+	lex_error(0, s);
 }
 
 
@@ -970,26 +1076,40 @@ lex_error(s sva_last)
  *	lex_warning - print a file location specific warning message
  *
  * SYNOPSIS
- *	void lex_warning(char *s, ...);
+ *	void lex_warning(char *s);
  *
  * DESCRIPTION
  *	Lex_warning prints a file location specific warning message.
  */
 
 void
-lex_warning(s sva_last)
+lex_warning(scp, s)
+	sub_context_ty	*scp;
 	char		*s;
-	sva_last_decl
 {
 	string_ty	*buffer;
-	va_list		ap;
+	int		need_to_delete;
+
+	if (scp)
+		need_to_delete = 0;
+	else
+	{
+		scp = sub_context_new();
+		need_to_delete = 1;
+	}
 
 	assert(root);
-	sva_init(ap, s);
-	buffer = str_vformat(s, ap);
-	va_end(ap);
-	error("%S: %d: warning: %S", root->l_name, root->l_line, buffer);
+	buffer = subst_intl(scp, s);
+
+	/* re-use the substitution context */
+	sub_var_set(scp, "File_Name", "%S", root->filename.logical);
+	sub_var_set(scp, "Number", "%ld", root->l_line);
+	sub_var_set(scp, "MeSsaGe", "%S", buffer);
+	error_intl(scp, i18n("$filname: $number: warning: $message"));
 	str_free(buffer);
+
+	if (need_to_delete)
+		sub_context_delete(scp);
 }
 
 
@@ -1062,7 +1182,13 @@ lex_trace(s sva_last)
 	if (cp > line && *cp == '\n')
 	{
 		*cp = 0;
-		trace_printf("%S: %d: %s\n", root->l_name, root->l_line, line);
+		trace_printf
+		(
+			"%S: %d: %s\n",
+			root->filename.logical,
+			root->l_line,
+			line
+		);
 		line[0] = 0;
 	}
 }
@@ -1162,13 +1288,41 @@ tokenize()
 		switch (c.m_type)
 		{
 		default:
-			fatal("illegal m_type %d (bug %d)", c.m_type, __LINE__);
+			fatal_raw("illegal m_type %d (bug %d)", c.m_type, __LINE__);
 
 		case EOF:
-			tok = 0; /* by definition of yacc */
+			if (!root->have_said_eof_token)
+			{
+				tok = FILE_BOUNDARY;
+				root->have_said_eof_token = 1;
+			}
+			else
+				tok = 0; /* by definition of yacc */
 			goto ret;
 
 		case NORMAL:
+			if (c.m_char == ':')
+			{
+				meta_ty	hold;
+
+				hold = c;
+				meta(&c);
+				if (c.m_char == ':' && c.m_type == NORMAL)
+				{
+					sa_open();
+					sa_char(':');
+					sa_char(':');
+					parse_lval.lv_word = sa_close();
+					state = 1;
+					tok = UNQUOTED_WORD;
+					last = c;
+					goto ret;
+				}
+				meta_undo(&c);
+				c = hold;
+			}
+			/* fall through... */
+
 		case SLOSHED:
 			{
 				int	 sloshed;
@@ -1181,8 +1335,8 @@ tokenize()
 				{
 					trace
 					((
-							"%s: %d: state %d, mode %s;\n",
-						root->l_name->str_text,
+						"%s: %d: state %d, mode %s;\n",
+						root->filename.logical->str_text,
 						root->l_line,
 						state,
 						mode_repn(mode)
@@ -1237,7 +1391,8 @@ tokenize()
 								/* don't change state */
 								lex_error
 								(
-							      "newline in quote"
+									0,
+							i18n("newline in quote")
 								);
 								break;
 
@@ -1251,7 +1406,7 @@ tokenize()
 						case '\'':
 							if (state != 12)
 							{
-								static		int		oldmode = LM_SQUOTE;
+								static int	oldmode = LM_SQUOTE;
 
 								oldmode = lex_mode(oldmode);
 								sa_goto(sa_mark()-1);
@@ -1263,7 +1418,7 @@ tokenize()
 						case '"':
 							if (state != 11)
 							{
-								static		int		oldmode = LM_DQUOTE;
+								static int	oldmode = LM_DQUOTE;
 
 								oldmode = lex_mode(oldmode);
 								sa_goto(sa_mark()-1);
@@ -1423,7 +1578,7 @@ tokenize()
 	trace
 	((
 		"%S: %d: state %d, mode %s;\n",
-		root->l_name,
+		root->filename.logical,
 		root->l_line,
 		state,
 		mode_repn(mode)
@@ -1456,7 +1611,7 @@ string_ty *
 lex_cur_file()
 {
 	assert(root);
-	return root->l_name;
+	return root->filename.logical;
 }
 
 
@@ -1500,7 +1655,7 @@ parse_lex()
 		{
 			if (!tok)
 			{
-				lex_error("unterminated conditional");
+				lex_error(0, i18n("unterminated conditional"));
 				break;
 			}
 			if (tok == WORD || tok == UNQUOTED_WORD)
@@ -1520,12 +1675,25 @@ parse_lex()
 				tok = *data;
 				str_free(parse_lval.lv_word);
 				parse_lval.lv_word = 0;
-				if (tok == COLON)
+				if (tok == COLON || tok == COLON2)
 				{
-					parse_lval.lv_position.pos_name = str_copy(root->l_name);
-					parse_lval.lv_position.pos_line = root->l_line;
+					parse_lval.lv_position.pos_name =
+						str_copy(root->filename.logical);
+					parse_lval.lv_position.pos_line =
+						root->l_line;
+					parse_lval.lv_position.multi =
+						(tok == COLON2);
+					tok = COLON;
 				}
-				if (tok == DATA)
+				else if (tok == SEMICOLON)
+				{
+					parse_lval.lv_position.pos_name =
+						str_copy(root->filename.logical);
+					parse_lval.lv_position.pos_line =
+						root->l_line;
+					parse_lval.lv_position.multi = 0;
+				}
+				else if (tok == DATA)
 				{
 					int	winged;
 					meta_ty	last;
@@ -1544,7 +1712,7 @@ parse_lex()
 						switch (c.m_type)
 						{
 						default:
-							fatal
+							fatal_raw
 							(
 						   "illegal m_type %d (bug %d)",
 								c.m_type,
@@ -1560,7 +1728,8 @@ parse_lex()
 							{
 								lex_error
 								(
-			       "the 'data' keyword must be the last on the line"
+									0,
+			 i18n("the 'data' keyword must be the last on the line")
 								);
 								winged = 1;
 							}
@@ -1607,11 +1776,15 @@ parse_lex()
  *	Intended solely for use by the yacc-generated compiler
  */
 
+static int hashline_lex_nwords;
+
 int
 hashline_lex()
 {
 	int		tok;
 	int		*data;
+	string_ty	*guess;
+	sub_context_ty	*scp;
 
 	trace(("hashline_lex()\n{\n"/*}*/));
 	tok = tokenize();
@@ -1623,17 +1796,64 @@ hashline_lex()
 
 	case UNQUOTED_WORD:
 		assert(hash_symtab);
-		data = symtab_query(hash_symtab, parse_lval.lv_word);
+		if (hashline_lex_nwords)
+		{
+			data = symtab_query(hash_symtab, parse_lval.lv_word);
+			if (data)
+			{
+				tok = *data;
+				str_free(parse_lval.lv_word);
+				parse_lval.lv_word = 0;
+			}
+			else
+				tok = WORD;
+			break;
+		}
+		data = symtab_query(hash_directive_symtab, parse_lval.lv_word);
 		if (data)
 		{
 			tok = *data;
 			str_free(parse_lval.lv_word);
 			parse_lval.lv_word = 0;
+			break;
 		}
-		else
-			tok = WORD;
+		guess = 0;
+		data =
+			symtab_query_fuzzy
+			(
+				hash_directive_symtab,
+				parse_lval.lv_word,
+				&guess
+			);
+		if (data)
+		{
+			scp = sub_context_new();
+			assert(guess);
+			sub_var_set(scp, "Name", "%S", parse_lval.lv_word);
+			sub_var_set(scp, "Guess", "%S", guess);
+			lex_error
+			(
+				scp,
+				i18n("unknown #$name directive, guess #$guess")
+			);
+			sub_context_delete(scp);
+			tok = *data;
+			/* DO NOT str_free guess */
+			str_free(parse_lval.lv_word);
+			parse_lval.lv_word = 0;
+			break;
+		}
+		scp = sub_context_new();
+		sub_var_set(scp, "Name", "%S", parse_lval.lv_word);
+		lex_error(scp, i18n("unknown #$name directive"));
+		sub_context_delete(scp);
+		str_free(parse_lval.lv_word);
+		parse_lval.lv_word = 0;
+		tok = HASH_JUNK;
 		break;
 	}
+	++hashline_lex_nwords;
+	assert(sizeof(hashline_lval) == sizeof(parse_lval));
 	memcpy(&hashline_lval, &parse_lval, sizeof(hashline_lval));
 	trace(("return %d;\n", tok));
 	trace((/*{*/"}\n"));
@@ -1641,9 +1861,44 @@ hashline_lex()
 }
 
 
+void
+hashline_lex_reset()
+{
+	hashline_lex_nwords = 0;
+}
+
+
 void
 lex_passing(n)
 	int		n;
 {
 	passing = n;
 }
+
+
+void
+lex_lino_set(line, file)
+	string_ty	*line;
+	string_ty	*file;
+{
+	long		n;
+
+	assert(root);
+	if (!root)
+		return;
+
+	n = atol(line->str_text);
+	if (n <= 0)
+	{
+		lex_error(0, i18n("#line needs positive decimal line number"));
+		return;
+	}
+	root->l_line = n;
+
+	assert(root->filename.logical);
+	if (file)
+	{
+		str_free(root->filename.logical);
+		root->filename.logical = str_copy(file);
+	}
+}
