@@ -1,6 +1,6 @@
 /*
  *	cook - file construction tool
- *	Copyright (C) 1991, 1992, 1993, 1994 Peter Miller.
+ *	Copyright (C) 1991, 1992, 1993, 1994, 1996, 1997, 1998 Peter Miller;
  *	All rights reserved.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -15,7 +15,7 @@
  *
  *	You should have received a copy of the GNU General Public License
  *	along with this program; if not, write to the Free Software
- *	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
  *
  * MANIFEST: functions to manage command line options
  *
@@ -35,21 +35,30 @@
  * after parse errors or interrupts have happened.
  */
 
-#include <ctype.h>
+#include <ac/ctype.h>
 #include <ac/limits.h>
 #include <ac/stddef.h>
-#include <stdio.h>
+#include <ac/stdio.h>
 #include <ac/stdlib.h>
 #include <ac/string.h>
 #include <ac/time.h>
 
-#include <option.h>
-#include <error.h>
-#include <os.h>
+#include <libdir.h>
 #include <mem.h>
+#include <option.h>
+#include <os.h>
+#include <progname.h>
+#include <trace.h>
 
 
+typedef struct flag_ty flag_ty;
+struct flag_ty
+{
+	unsigned long o_flag[OPTION_max];
+};
+
 option_ty option;
+static flag_ty flag;
 
 
 /*
@@ -75,9 +84,9 @@ option_set(o, level, state)
 {
 	assert((int)o >= 0 && (int)o < (int)OPTION_max);
 	if (state)
-		option.o_flag[(size_t)o] |= 3 << (2 * (int)level);
+		flag.o_flag[(size_t)o] |= 3L << (2 * (int)level);
 	else
-		option.o_flag[(size_t)o] |= 1 << (2 * (int)level);
+		flag.o_flag[(size_t)o] |= 1L << (2 * (int)level);
 }
 
 
@@ -102,7 +111,7 @@ option_already(o, level)
 	option_level_ty	level;
 {
 	assert((int)o >= 0 && (int)o < (int)OPTION_max);
-	return ((option.o_flag[(size_t)o] >> (2 * (int)level)) & 1) != 0;
+	return ((flag.o_flag[(size_t)o] >> (2 * (int)level)) & 1) != 0;
 }
 
 
@@ -127,7 +136,7 @@ option_undo(o, level)
 	option_level_ty	level;
 {
 	assert((int)o >= 0 && (int)o < (int)OPTION_max);
-	option.o_flag[(size_t)o] &= ~(3 << (2 * (int)level));
+	flag.o_flag[(size_t)o] &= ~(3L << (2 * (int)level));
 }
 
 
@@ -174,13 +183,13 @@ option_undo_level(level)
 
 int
 option_test(o)
-	option_number_ty	o;
+	option_number_ty o;
 {
-	unsigned	*op;
-	unsigned	mask;
+	unsigned long	*op;
+	unsigned long	mask;
 
 	assert((int)o >= 0 && (int)o < (int)OPTION_max);
-	op = &option.o_flag[(size_t)o];
+	op = &flag.o_flag[(size_t)o];
 	mask = *op & 0x55555555;
 	mask &= -mask; /* get LSB */
 	return (*op & (mask << 1)) != 0;
@@ -214,18 +223,23 @@ option_tidy_up()
 	/*
 	 * user's library
 	 */
+	trace(("tidy up\n"));
 	s = os_accdir();
 	assert(s);
-	s1 = str_format("%S/.%s", s, progname);
+	s1 = str_format("%S/.%s", s, progname_get());
 	str_free(s);
-	wl_append_unique(&option.o_search_path, s1);
+	string_list_append_unique(&option.o_search_path, s1);
 	str_free(s1);
 
 	/*
 	 * cook's library
+	 *	architecture-specific, then architecture-neutral
 	 */
-	s = str_from_c(LIBDIR);
-	wl_append_unique(&option.o_search_path, s);
+	s = str_from_c(library_directory_get());
+	string_list_append_unique(&option.o_search_path, s);
+	str_free(s);
+	s = str_from_c(data_directory_get());
+	string_list_append_unique(&option.o_search_path, s);
 	str_free(s);
 
 	if (!option.o_book)
@@ -253,7 +267,7 @@ option_tidy_up()
 
 			strcpy(format, name[j]);
 			floop:
-			s = str_format(format, progname);
+			s = str_format(format, progname_get());
 			switch (os_exists(s))
 			{
 			case -1:
@@ -294,9 +308,12 @@ option_tidy_up()
 	}
 
 	option_set(OPTION_ACTION, OPTION_LEVEL_DEFAULT, 1);
+	option_set(OPTION_CASCADE, OPTION_LEVEL_DEFAULT, 1);
 	option_set(OPTION_LOGGING, OPTION_LEVEL_DEFAULT, 1);
 	option_set(OPTION_TERMINAL, OPTION_LEVEL_DEFAULT, 1);
 	option_set(OPTION_STRIP_DOT, OPTION_LEVEL_DEFAULT, 1);
+	option_set(OPTION_INCLUDE_COOKED, OPTION_LEVEL_DEFAULT, 1);
+	option_set(OPTION_FINGERPRINT_WRITE, OPTION_LEVEL_DEFAULT, 1);
 }
 
 
@@ -324,3 +341,26 @@ option_set_errors()
 	option_set(OPTION_METER, OPTION_LEVEL_ERROR, 0);
 	option_set(OPTION_PERSEVERE, OPTION_LEVEL_ERROR, 0);
 }
+
+
+void *
+option_flag_state_get()
+{
+	flag_ty		*fp;
+
+	fp = mem_alloc(sizeof(flag_ty));
+	*fp = flag;
+	return fp;
+}
+
+
+void
+option_flag_state_set(p)
+	void		*p;
+{
+	flag_ty		*fp;
+
+	fp = p;
+	flag = *fp;
+	mem_free(p);
+}
