@@ -1,7 +1,7 @@
 /*
  *	cook - file construction tool
- *	Copyright (C) 1990, 1991, 1992, 1993, 1994 Peter Miller.
- *	All rights reserved.
+ *	Copyright (C) 1990, 1991, 1992, 1993, 1994, 1995, 1996, 1997
+ *	Peter Miller; All rights reserved.
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -15,7 +15,7 @@
  *
  *	You should have received a copy of the GNU General Public License
  *	along with this program; if not, write to the Free Software
- *	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
  *
  * MANIFEST: interface definition for cook/option.c
  */
@@ -26,7 +26,7 @@
 #include <main.h>
 
 #include <str.h>
-#include <word.h>
+#include <str_list.h>
 
 /*
  * option levels, highest to lowest
@@ -47,24 +47,39 @@ typedef enum option_level_ty option_level_ty;
 
 enum option_number_ty
 {
-	OPTION_PERSEVERE,	/* keep trying if have errors */
-	OPTION_SILENT,		/* do not echo any command */
-	OPTION_ERROK,		/* ignore error returns from commands */
 	OPTION_ACTION,		/* do not execute the command */
-	OPTION_TOUCH,		/* do not execute the command, just touch */
-	OPTION_FORCE,		/* always execute the commands */
-	OPTION_PRECIOUS,	/* do not delete failed targets */
-	OPTION_TERMINAL,	/* enable tty output when logging */
-	OPTION_TRACE,		/* emit tracing information */
-	OPTION_CMDFILE,		/* generate a command file */
-	OPTION_METER,		/* meter each command */
-	OPTION_LOGGING,
 	OPTION_BOOK,
-	OPTION_INVALIDATE_STAT_CACHE,
-	OPTION_STAR,		/* emit progress stars */
+	OPTION_CASCADE,		/* do (not) cascade ingredients */
+	OPTION_CMDFILE,		/* generate a command file */
+	OPTION_DISASSEMBLE,	/* undocumented: disassemble opcode lists after compilation */
+	OPTION_ERROK,		/* ignore error returns from commands */
 	OPTION_FINGERPRINT,	/* remember file fingerprints */
-	OPTION_UPDATE,		/* update utime for consistency */
+	OPTION_FINGERPRINT_WRITE, /* preserve fingerprints if taken */
+	OPTION_FORCE,		/* always execute the commands */
+	OPTION_IMPLICIT_ALLOWED, /* implicit recipes may be used */
+	OPTION_INCLUDE_COOKED,	/* cook the include-cooked include files */
+	OPTION_INVALIDATE_STAT_CACHE,
+	OPTION_LOGGING,
+	OPTION_METER,		/* meter each command */
+	OPTION_MKDIR,		/* make directories of targets */
+	OPTION_PERSEVERE,	/* keep trying if have errors */
+	OPTION_PRECIOUS,	/* do not delete failed targets */
+	OPTION_REASON,		/* emit inference debugging commentary */
+	OPTION_RECURSE,		/* allow target recursion loops */
+	OPTION_SHALLOW,		/* recipe targets are to be shallow on search_list */
+	OPTION_SILENT,		/* do not echo any command */
+	OPTION_STAR,		/* emit progress stars */
 	OPTION_STRIP_DOT,	/* strip leading ./ from paths */
+	OPTION_TERMINAL,	/* enable tty output when logging */
+	OPTION_TOUCH,		/* do not execute the command, just touch */
+	OPTION_UNLINK,		/* remove targets before running rule body */
+	OPTION_UPDATE,		/* update utime for consistency */
+	OPTION_UPDATE_MAX,	/* update utime for consistency - backwards! */
+
+	/*
+	 * If you add to this list, make sure you also add the option to
+	 * the list in cook/builtin/options.c
+	 */
 	OPTION_max
 };
 typedef enum option_number_ty option_number_ty;
@@ -72,16 +87,16 @@ typedef enum option_number_ty option_number_ty;
 typedef struct option_ty option_ty;
 struct option_ty
 {
-	unsigned	o_flag[OPTION_max];
-	wlist		o_target;
+	string_list_ty	o_target;
 	string_ty	*o_book;
 	string_ty	*o_logfile;
-	wlist		o_search_path;
-	wlist		o_vardef;
+	string_list_ty	o_search_path;
+	string_list_ty	o_vardef;
+	int		pairs;
+	int		script;
 };
 
 extern	option_ty	option;
-extern	char		*progname;
 
 int option_already _((option_number_ty, option_level_ty));
 int option_test _((option_number_ty));
@@ -91,4 +106,7 @@ void option_undo_level _((option_level_ty));
 void option_set_errors _((void));
 void option_tidy_up _((void));
 
+void *option_flag_state_get _((void));
+void option_flag_state_set _((void *));
+
 #endif /* OPTION_H */
