@@ -1,6 +1,6 @@
 /*
  *	cook - file construction tool
- *	Copyright (C) 1991, 1992, 1993, 1994 Peter Miller.
+ *	Copyright (C) 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998 Peter Miller;
  *	All rights reserved.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -15,7 +15,7 @@
  *
  *	You should have received a copy of the GNU General Public License
  *	along with this program; if not, write to the Free Software
- *	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
  *
  * MANIFEST: functions to isolate operating system interface
  */
@@ -25,23 +25,25 @@
 #include <ac/stdlib.h>
 #include <ac/limits.h>
 #include <errno.h>
-#include <stdio.h>
+#include <ac/stdio.h>
 #include <signal.h>
 #include <ac/time.h>
 #include <ac/unistd.h>
 
 #include <archive.h>
+#include <sys/types.h>
 #include <sys/stat.h>
 #include <sys/param.h>
-#include <utime.h>
+#include <sys/wait.h>
+#include <ac/utime.h>
 
-#include <error.h>
+#include <error_intl.h>
 #include <mem.h>
 #include <option.h>
 #include <os.h>
 #include <stat.cache.h>
 #include <trace.h>
-#include <word.h>
+#include <str_list.h>
 
 #ifdef HAVE_GETRUSAGE
 #include <sys/resource.h>
@@ -69,14 +71,87 @@ time_t
 os_mtime_oldest(path)
 	string_ty	*path;
 {
-	return stat_cache_oldest(path);
+	return stat_cache_oldest(path, 1);
 }
 
 time_t
 os_mtime_newest(path)
 	string_ty	*path;
 {
-	return stat_cache_newest(path);
+	return stat_cache_newest(path, 1);
+}
+
+
+static void adjust_message _((string_ty *, long));
+
+static void
+adjust_message(path, nsec)
+	string_ty	*path;
+	long		nsec;
+{
+	int		direction;
+	string_ty	*buffer;
+	sub_context_ty	*scp;
+
+	/*
+	 * work out the direction of adjustment
+	 */
+	trace(("adjust_message(nsec = %ld)\n{\n", nsec));
+	if (nsec >= 0) 
+		direction = 1;
+	else
+	{
+		nsec = -nsec;
+		direction = -1;
+	}
+
+	/*
+	 * work out the units of adjustment
+	 */
+	scp = sub_context_new();
+	if (nsec >= (365L * 24 * 60 * 60))
+	{
+		sub_var_set(scp, "Number", "%4.2f", nsec / (double)(365L * 24 * 60 * 60));
+		buffer = subst_intl(scp, i18n("$number years"));
+	}
+	else if (nsec >= (24L * 60 * 60))
+	{
+		sub_var_set(scp, "Number1", "%ld", nsec / (24L * 60 * 60));
+		sub_var_set(scp, "Number2", "%ld", (nsec / (60 * 60)) % 24);
+		buffer = subst_intl(scp, i18n("$number1 days $number2 hours"));
+	}
+	else if (nsec >= (60 * 60))
+	{
+		sub_var_set(scp, "Number1", "%ld", nsec / (60 * 60));
+		sub_var_set(scp, "Number2", "%ld", (nsec / 60) % 60);
+		buffer = subst_intl(scp, i18n("$number1 hours $number2 minutes"));
+	}
+	else if (nsec >= 60)
+	{
+		sub_var_set(scp, "Number1", "%ld", nsec / 60);
+		sub_var_set(scp, "Number2", "%ld", nsec % 60);
+		buffer = subst_intl(scp, i18n("$number1 minutes $number2 seconds"));
+	}
+	else
+	{
+		sub_var_set(scp, "Number", "%ld", nsec);
+		buffer = subst_intl(scp, i18n("$number seconds"));
+	}
+
+	/*
+	 * print the message
+	 * (re-use substitution context)
+	 */
+	trace(("mark\n"));
+	sub_var_set(scp, "File_Name", "%S", path);
+	sub_var_set(scp, "Number", "%S", buffer);
+	if (direction < 0)
+		error_intl(scp, i18n("adjusting \"$filename\" back $number"));
+	else
+		error_intl(scp, i18n("adjusting \"$filename\" forward $number"));
+	str_free(buffer);
+	sub_context_delete(scp);
+	trace(("}\n"));
 }
 
 
@@ -105,42 +180,92 @@ os_mtime_adjust(path, min_age)
 	time_t		mtime;
 	int		err;
 
-	if (option_test(OPTION_UPDATE))
+	trace(("os_mtime_adjust()\n{\n"));
+	if (option_test(OPTION_UPDATE) && option_test(OPTION_ACTION))
 	{
 		stat_cache_clear(path);
 		mtime = os_mtime_newest(path);
 		if (mtime < 0)
+		{
+			trace(("return -1;\n"));
+			trace(("}\n"));
 			return -1;
+		}
 		if (mtime)
 		{
-			if (mtime < min_age)
+			if
+			(
+				option_test(OPTION_UPDATE_MAX)
+			?
+				mtime != min_age
+			:
+				mtime < min_age
+			)
 			{
 				struct utimbuf	ut;
 
 				if (!option_test(OPTION_SILENT))
-				{
-					long		nsec;
-
-					nsec = min_age - mtime;
-					error
-					(
-					"adjusting \"%s\" forward %ld second%s",
-						path->str_text,
-						nsec,
-						(nsec == 1 ? "" : "s")
-					);
-				}
+					adjust_message(path, (long)min_age - (long)mtime);
 				ut.modtime = min_age;
 				ut.actime = min_age;
 				err = utime(path->str_text, &ut);
 				if (err && errno == ENOENT)
 					err = archive_utime(path, &ut);
+				if (err && errno == EPERM)
+				{
+					sub_context_ty	*scp;
+
+					/*
+					 * The only way to get this
+					 * error message is if the
+					 * recipe body invoked sudo (or
+					 * similar) to change the owner.
+					 *
+					 * They did it to themselves, so
+					 * just give them a warning.
+					 */
+					scp = sub_context_new();
+					sub_errno_set(scp);
+					sub_var_set(scp, "File_Name", "%S", path);
+					error_intl
+					(
+						scp,
+			   i18n("warning: when adjusting \"$filename\": $errno")
+					);
+					trace(("return 0;\n"));
+					trace(("}\n"));
+					return 0;
+				}
 				if (err)
 				{
-					nerror("utime(\"%s\")", path->str_text);
+					sub_context_ty	*scp;
+
+					scp = sub_context_new();
+					sub_errno_set(scp);
+					sub_var_set(scp, "File_Name", "%S", path);
+					error_intl
+					(
+						scp,
+						i18n("utime $filename: $errno")
+					);
+					sub_context_delete(scp);
 					option_set_errors();
+					trace(("return -1;\n"));
+					trace(("}\n"));
 					return -1;
 				}
+#if defined(sun) || defined(__sun__)
+				else
+				{
+					/*
+					 * Solaris is brain dead.
+					 * The inode times are not
+					 * updated until the next sync.
+					 * So we'll help it along.
+					 */
+					sync();
+				}
+#endif
 				stat_cache_set(path, min_age, 1);
 			}
 		}
@@ -163,6 +288,8 @@ os_mtime_adjust(path, min_age)
 			min_age = mtime;
 		stat_cache_set(path, min_age, 0);
 	}
+	trace(("return 0;\n"));
+	trace(("}\n"));
 	return 0;
 }
 
@@ -229,15 +356,31 @@ os_touch(path)
 		err = archive_utime(path, &ut);
 	if (err)
 	{
+		sub_context_ty	*scp;
+		
 		if (errno == ENOENT)
 		{
 			stat_cache_clear(path);
 			return 0;
 		}
-		nerror("utime(\"%s\")", path->str_text);
+		scp = sub_context_new();
+		sub_errno_set(scp);
+		sub_var_set(scp, "File_Name", "%S", path);
+		error_intl(scp, i18n("utime $filename: $errno"));
+		sub_context_delete(scp);
 		option_set_errors();
 		return -1;
 	}
+#if defined(sun) || defined(__sun__)
+	else
+	{
+		/*
+		 * Solaris is brain dead.  The inode times are not
+		 * updated until the next sync.  So we'll help it along.
+		 */
+		sync();
+	}
+#endif
 	stat_cache_set(path, ut.modtime, 1);
 	return 0;
 }
@@ -259,14 +402,33 @@ os_touch(path)
  */
 
 int
-os_delete(path)
+os_delete(path, echo)
 	string_ty	*path;
+	int		echo;
 {
-	if (unlink(path->str_text) && errno != ENOENT)
+	if (unlink(path->str_text))
 	{
-		nerror("unlink(\"%s\")", path->str_text);
-		option_set_errors();
-		return -1;
+		if (errno != ENOENT)
+		{
+			sub_context_ty	*scp;
+
+			scp = sub_context_new();
+			sub_errno_set(scp);
+			sub_var_set(scp, "File_Name", "%S", path);
+			error_intl(scp, i18n("unlink $filename: $errno"));
+			sub_context_delete(scp);
+			option_set_errors();
+			return -1;
+		}
+	}
+	else if (echo)
+	{
+		sub_context_ty	*scp;
+
+		scp = sub_context_new();
+		sub_var_set(scp, "File_Name", "%S", path);
+		error_intl(scp, i18n("rm $filename"));
+		sub_context_delete(scp);
 	}
 
 	/*
@@ -278,135 +440,6 @@ os_delete(path)
 }
 
 
-/*
- * NAME
- *	signal_name - find it
- *
- * SYNOPSIS
- *	char *signal_name(int n);
- *
- * DESCRIPTION
- *	The signal_name function is used to find the name of a signal from its
- *	number.
- *
- * RETURNS
- *	char *: pointer to the signal name.
- *
- * CAVEAT
- *	The signal name may not be written on.  Subsequent calls may alter the
- *	area pointed to.
- */
-
-char *
-signal_name(n)
-	int		n;
-{
-	static char	buffer[16];
-
-	switch (n)
-	{
-#ifdef SIGHUP
-	case SIGHUP:
-		return "hang up [SIGHUP]";
-#endif /* SIGHUP */
-
-#ifdef SIGINT
-	case SIGINT:
-		return "user interrupt [SIGINT]";
-#endif /* SIGINT */
-
-#ifdef SIGQUIT
-	case SIGQUIT:
-		return "user quit [SIGQUIT]";
-#endif /* SIGQUIT */
-
-#ifdef SIGILL
-	case SIGILL:
-		return "illegal instruction [SIGILL]";
-#endif /* SIGILL */
-
-#ifdef SIGTRAP
-	case SIGTRAP:
-		return "trace trap [SIGTRAP]";
-#endif /* SIGTRAP */
-
-#ifdef SIGIOT
-	case SIGIOT:
-		return "abort [SIGIOT]";
-#endif /* SIGIOT */
-
-#ifdef SIGEMT
-	case SIGEMT:
-		return "EMT instruction [SIGEMT]";
-#endif /* SIGEMT */
-
-#ifdef SIGFPE
-	case SIGFPE:
-		return "floating point exception [SIGFPE]";
-#endif /* SIGFPE */
-
-#ifdef SIGKILL
-	case SIGKILL:
-		return "kill [SIGKILL]";
-#endif /* SIGKILL */
-
-#ifdef SIGBUS
-	case SIGBUS:
-		return "bus error [SIGBUS]";
-#endif /* SIGBUS */
-
-#ifdef SIGSEGV
-	case SIGSEGV:
-		return "segmentation violation [SIGSEGV]";
-#endif /* SIGSEGV */
-
-#ifdef SIGSYS
-	case SIGSYS:
-		return "bad argument to system call [SIGSYS]";
-#endif /* SIGSYS */
-
-#ifdef SIGPIPE
-	case SIGPIPE:
-		return "write on a pipe with no one to read it [SIGPIPE]";
-#endif /* SIGPIPE */
-
-#ifdef SIGALRM
-	case SIGALRM:
-		return "alarm clock [SIGALRM]";
-#endif /* SIGALRM */
-
-#ifdef SIGTERM
-	case SIGTERM:
-		return "software termination [SIGTERM]";
-#endif /* SIGTERM */
-
-#ifdef SIGUSR1
-	case SIGUSR1:
-		return "user defined signal one [SIGUSR1]";
-#endif /* SIGUSR1 */
-
-#ifdef SIGUSR2
-	case SIGUSR2:
-		return "user defined signal two [SIGUSR2]";
-#endif /* SIGUSR2 */
-
-#ifdef SIGCLD
-	case SIGCLD:
-		return "death of child [SIGCLD]";
-#endif /* SIGCLD */
-
-#ifdef SIGPWR
-	case SIGPWR:
-		return "power failure [SIGPWR]";
-#endif /* SIGPWR */
-
-	default:
-		sprintf(buffer, "signal %d", n);
-		return buffer;
-	}
-}
-
-
 /*
  * NAME
  *	exit_status - pretty print
@@ -433,6 +466,7 @@ exit_status(cmd, status, errok)
 	int		errok;
 {
 	int		a, b, c;
+	sub_context_ty	*scp;
 
 	a = (status >> 8) & 0xFF;
 	b = (status >> 7) & 1;
@@ -444,20 +478,36 @@ exit_status(cmd, status, errok)
 		 * process was stopped,
 		 * since we didn't do it, treat it as an error
 		 */
-		error("%s: stopped", cmd);
+		scp = sub_context_new();
+		sub_var_set(scp, "File_Name", "%s", cmd);
+		error_intl(scp, i18n("command $filename: stopped"));
+		sub_context_delete(scp);
 		return 1;
 
 	case 0:
 		/* normal termination */
 		if (a)
 		{
-			error
-			(
-				"%s: exit status: %d%s",
-				cmd,
-				a,
-				(errok ? " (ignored)" : "")
-			);
+			scp = sub_context_new();
+			sub_var_set(scp, "File_Name", "%s", cmd);
+			sub_var_set(scp, "Number", "%d", a);
+			if (errok)
+			{
+				error_intl
+				(
+					scp,
+			i18n("command $filename: exit status $number (ignored)")
+				);
+			}
+			else
+			{
+				error_intl
+				(
+					scp,
+				  i18n("command $filename: exit status $number")
+				);
+			}
+			sub_context_delete(scp);
 			if (errok)
 				return 0;
 			return 1;
@@ -468,13 +518,25 @@ exit_status(cmd, status, errok)
 		/*
 		 * process dies from unhandled condition
 		 */
-		error
-		(
-			"%s: terminated by %s%s",
-			cmd,
-			signal_name(c),
-			(b ? " (core dumped)" : "")
-		);
+		scp = sub_context_new();
+		sub_var_set(scp, "File_Name", "%s", cmd);
+		sub_var_set(scp, "Name", "%s", strsignal(c));
+		if (b)
+		{
+			error_intl
+			(
+				scp,
+		    i18n("command $filename: terminated by $name (core dumped)")
+			);
+		}
+		else
+		{
+			error_intl
+			(
+				scp,
+				i18n("command $filename: terminated by $name")
+			);
+		}
 		return 1;
 	}
 }
@@ -485,7 +547,7 @@ exit_status(cmd, status, errok)
  *	execute - do a command
  *
  * SYNOPSIS
- *	int execute(wlist *cmd, int fd, int errok);
+ *	int execute(string_list_ty *cmd, int fd, int errok);
  *
  * DESCRIPTION
  *	The execute function is used to execute the command in the word list.
@@ -498,11 +560,11 @@ exit_status(cmd, status, errok)
  * CAVEAT
  */
 
-static int execute _((wlist *, int, int));
+static int execute _((string_list_ty *, int, int));
 
 static int
 execute(cmd, fd, errok)
-	wlist		*cmd;
+	string_list_ty	*cmd;
 	int		fd;
 	int		errok;
 {
@@ -512,42 +574,68 @@ execute(cmd, fd, errok)
 	static char	**argv;
 	static size_t	argvlen;
 	int		j;
+	sub_context_ty	*scp;
 
 	if (!argv)
 	{
-		argvlen = cmd->wl_nwords + 1;
+		argvlen = cmd->nstrings + 1;
 		argv = mem_alloc(argvlen * sizeof(char*));
 	}
 	else
 	{
-		if (argvlen < cmd->wl_nwords + 1)
+		if (argvlen < cmd->nstrings + 1)
 		{
-			argvlen = cmd->wl_nwords + 1;
+			argvlen = cmd->nstrings + 1;
 			argv = mem_change_size(argv, argvlen * sizeof(char *));
 		}
 	}
-	for (j = 0; j < cmd->wl_nwords; ++j)
-		argv[j] = cmd->wl_word[j]->str_text;
-	argv[cmd->wl_nwords] = 0;
+	for (j = 0; j < cmd->nstrings; ++j)
+		argv[j] = cmd->string[j]->str_text;
+	argv[cmd->nstrings] = 0;
 	switch (child = fork())
 	{
 	case -1:
-		nerror("fork()");
+		scp = sub_context_new();
+		sub_errno_set(scp);
+		error_intl(scp, i18n("fork(): $errno"));
+		sub_context_delete(scp);
 		return -1;
 
 	case 0:
 		if (fd >= 0)
 		{
 			if (close(0) && errno != EBADF)
-				nfatal("close(0)");
+			{
+				string_ty	*fn0;
+				int		err;
+
+				err = errno;
+				scp = sub_context_new();
+				fn0 = subst_intl(scp, "standard input");
+				/* re-use substitution context */
+				sub_errno_setx(scp, err);
+				sub_var_set(scp, "File_Name", "%S", fn0);
+				fatal_intl(scp, i18n("close $filename: $errno"));
+				sub_context_delete(scp);
+				str_free(fn0);
+			}
 			if (dup(fd) < 0)
-				nfatal("dup()");
+			{
+				scp = sub_context_new();
+				sub_errno_set(scp);
+				fatal_intl(scp, i18n("dup(): $errno"));
+				/* NOTREACHED */
+			}
 		}
 		if (argv[0][0] == '/')
 			execv(argv[0], argv);
 		else
 			execvp(argv[0], argv);
-		nfatal("exec(\"%s\")", argv[0]);
+		scp = sub_context_new();
+		sub_errno_set(scp);
+		sub_var_set(scp, "File_Name", "%s", argv[0]);
+		fatal_intl(scp, i18n("exec $filename: $errno"));
+		/* NOTREACHED */
 
 	default:
 		for (;;)
@@ -557,7 +645,10 @@ execute(cmd, fd, errok)
 				return exit_status(argv[0], status, errok);
 			if (pid < 0 && errno != EINTR)
 			{
-				nerror("wait()");
+				scp = sub_context_new();
+				sub_errno_set(scp);
+				error_intl(scp, i18n("wait(): $errno"));
+				sub_context_delete(scp);
 				option_set_errors();
 				return -1;
 			}
@@ -571,7 +662,7 @@ execute(cmd, fd, errok)
  *	os_execute - execute a command
  *
  * SYNOPSIS
- *	int os_execute(wlist *args, string_ty *input, int errok);
+ *	int os_execute(string_list_ty *args, string_ty *input, int errok);
  *
  * DESCRIPTION
  *	Os_execute performs the given command.
@@ -582,7 +673,7 @@ execute(cmd, fd, errok)
 
 int
 os_execute(args, input, errok)
-	wlist		*args;
+	string_list_ty	*args;
 	string_ty	*input;
 	int		errok;
 {
@@ -592,7 +683,7 @@ os_execute(args, input, errok)
 	int		retval;
 
 	assert(args);
-	assert(args->wl_nwords > 0);
+	assert(args->nstrings > 0);
 
 	fp = 0;
 	if (input)
@@ -602,10 +693,16 @@ os_execute(args, input, errok)
 		 * so write it out to a file, and then redirect the input.
 		 */
 		tmpnam(iname);
-		fp = fopen(iname, "w");
+		fp = fopen(iname, "w+");
 		if (!fp)
 		{
-			nerror("create %s", iname);
+			sub_context_ty	*scp;
+
+			scp = sub_context_new();
+			sub_errno_set(scp);
+			sub_var_set(scp, "File_Name", "%s", iname);
+			error_intl(scp, i18n("open $filename: $errno"));
+			sub_context_delete(scp);
 			exec_fails:
 			option_set_errors();
 			retval = -1;
@@ -613,52 +710,64 @@ os_execute(args, input, errok)
 		}
 		if (unlink(iname))
 		{
-			nerror("unlink(\"%s\")", iname);
+			sub_context_ty	*scp;
+
+			scp = sub_context_new();
+			sub_errno_set(scp);
+			sub_var_set(scp, "File_Name", "%s", iname);
+			error_intl(scp, i18n("unlink $filename: $errno"));
+			sub_context_delete(scp);
 			goto exec_fails;
 		}
 		fputs(input->str_text, fp);
 		if (ferror(fp) || fseek(fp, 0L, 0))
 		{
-			nerror("write %s", iname);
+			sub_context_ty	*scp;
+
+			scp = sub_context_new();
+			sub_errno_set(scp);
+			sub_var_set(scp, "File_Name", "%s", iname);
+			error_intl(scp, i18n("write $filename: $errno"));
+			sub_context_delete(scp);
 			goto exec_fails;
 		}
 	}
 
-	for (j = 1; j < args->wl_nwords; j++)
+	for (j = 1; j < args->nstrings; j++)
 	{
 		char	*s;
 
-		s = args->wl_word[j]->str_text;
+		s = args->string[j]->str_text;
 		while (*s)
 		{
 			if (strchr("#|=^();&<>*?[]:$`'\"\\\n", *s))
 			{
 				string_ty	*str;
-				wlist		newcmd;
+				string_list_ty	newcmd;
 				char		*cp;
 
 				cp = getenv("SHELL");
 				if (!cp || !*cp)
 					cp = CONF_SHELL;
-				wl_zero(&newcmd);
+				string_list_constructor(&newcmd);
 				str = str_from_c(cp);
-				wl_append(&newcmd, str);
+				string_list_append(&newcmd, str);
 				str_free(str);
 				if (option_test(OPTION_ERROK))
 					str = str_from_c("-c");
 				else
 					str = str_from_c("-ce");
-				wl_append(&newcmd, str);
+				string_list_append(&newcmd, str);
 				str_free(str);
 				str =
 					wl2str
 					(
 						args,
 						0,
-						args->wl_nwords - 1,
+						args->nstrings - 1,
 						(char *)0
 					);
-				wl_append(&newcmd, str);
+				string_list_append(&newcmd, str);
 				str_free(str);
 				retval =
 					execute
@@ -667,7 +776,7 @@ os_execute(args, input, errok)
 						(input ? fileno(fp) : -1),
 						errok
 					);
-				wl_free(&newcmd);
+				string_list_destructor(&newcmd);
 				if (input)
 					fclose(fp);
 				return retval;
@@ -703,7 +812,20 @@ os_exists(path)
 {
 	time_t		mtime;
 
-	mtime = stat_cache_newest(path);
+	mtime = stat_cache_newest(path, 1);
+	if (mtime < 0)
+		return -1;
+	return (mtime != 0);
+}
+
+
+int
+os_exists_symlink(path)
+	string_ty	*path;
+{
+	time_t		mtime;
+
+	mtime = stat_cache_newest(path, 0);
 	if (mtime < 0)
 		return -1;
 	return (mtime != 0);
@@ -742,7 +864,16 @@ os_accdir()
 		cp = getenv(home);
 		if (!cp)
 		{
-			error("%s: environment variable not defined", home);
+			sub_context_ty	*scp;
+
+			scp = sub_context_new();
+			sub_var_set(scp, "Name", "%s", home);
+			error_intl
+			(
+				scp,
+			       i18n("the $name environment variable is not set")
+			);
+			sub_context_delete(scp);
 			option_set_errors();
 			return 0;
 		}
@@ -782,7 +913,12 @@ os_curdir()
 
 		if (!getcwd(buffer, sizeof(buffer)))
 		{
-			nerror("getcwd");
+			sub_context_ty	*scp;
+
+			scp = sub_context_new();
+			sub_errno_set(scp);
+			error_intl(scp, i18n("getcwd: $errno"));
+			sub_context_delete(scp);
 			option_set_errors();
 			return 0;
 		}
@@ -821,9 +957,9 @@ os_pathname(path)
 	int		c;
 	int		found;
 #ifdef S_IFLNK
-	char		link[2000];
+	char		linkval[2000];
 	int		nbytes;
-	wlist		loop;
+	string_list_ty	loop;
 	string_ty	*s;
 #endif
 
@@ -856,7 +992,7 @@ os_pathname(path)
 	opos = 0;
 	found = 0;
 #ifdef S_IFLNK
-	wl_zero(&loop);
+	string_list_constructor(&loop);
 #endif
 	while (!found)
 	{
@@ -935,15 +1071,20 @@ os_pathname(path)
 		 */
 #ifdef S_IFLNK
 		s = str_n_from_c(tmp, opos);
-		if (wl_member(&loop, s))
+		if (string_list_member(&loop, s))
 		{
-			fatal
+			sub_context_ty	*scp;
+
+			scp = sub_context_new();
+			sub_var_set(scp, "File_Name", "%S", s);
+			fatal_intl
 			(
-				"symbolic link loop \"%s\" detected",
-				s->str_text
+				scp,
+			       i18n("symbolic link loop \"$filename\" detected")
 			);
+			/* NOTREACHED */
 		}
-		nbytes = readlink(s->str_text, link, sizeof(link) - 1);
+		nbytes = readlink(s->str_text, linkval, sizeof(linkval) - 1);
 		if (nbytes < 0)
 		{
 			/*
@@ -957,7 +1098,19 @@ os_pathname(path)
 			&&
 				errno != ENOENT
 			)
-				nfatal("readlink(\"%s\")", s->str_text);
+			{
+				sub_context_ty	*scp;
+
+				scp = sub_context_new();
+				sub_errno_set(scp);
+				sub_var_set(scp, "File_Name", "%S", s);
+				fatal_intl
+				(
+					scp,
+					i18n("readlink $filename: $errno")
+				);
+				/* NOTREACHED */
+			}
 			not_a_symlink:
 			str_free(s);
 		}
@@ -967,24 +1120,29 @@ os_pathname(path)
 
 			if (nbytes == 0)
 			{
-				fatal
+				sub_context_ty	*scp;
+
+				scp = sub_context_new();
+				sub_var_set(scp, "File_Name", "%S", s);
+				fatal_intl
 				(
-					"readlink(\"%s\") returned \"\"",
-					s->str_text
+					scp,
+				    i18n("readlink \"$filename\" returned \"\"")
 				);
+				/* NOTREACHED */
 			}
-			link[nbytes] = 0;
+			linkval[nbytes] = 0;
 
 			/*
 			 * ignore auto-mounter temporary mount links,
 			 * they will change with each mount.
 			 */
-			if (!strncmp(link, "/tmp_mnt/", 9))
+			if (!strncmp(linkval, "/tmp_mnt/", 9))
 				goto not_a_symlink;
 
-			wl_append(&loop, s);
+			string_list_append(&loop, s);
 			str_free(s);
-			if (link[0] == '/')
+			if (linkval[0] == '/')
 				tmp[1] = 0;
 			else
 			{
@@ -997,7 +1155,7 @@ os_pathname(path)
 				(
 					"%s/%s/%s",
 					tmp,
-					link,
+					linkval,
 					path->str_text + ipos
 				);
 			str_free(path);
@@ -1021,7 +1179,7 @@ os_pathname(path)
 		tmp[opos++] = c;
 	}
 #ifdef S_IFLNK
-	wl_free(&loop);
+	string_list_destructor(&loop);
 #endif
 	str_free(path);
 	assert(opos >= 1);
@@ -1118,163 +1276,60 @@ os_dirname(path)
 }
 
 
-/*
- * NAME
- *	os_meter_grab - get metering snapshot
- *
- * SYNOPSIS
- *	void os_meter_grab(grab*);
- *
- * DESCRIPTION
- *	The os_meter_gram function is used to take a snapshot of metering
- *	information.
- *
- * RETURNS
- *	void
- */
+#ifdef HAVE_PATHCONF
 
-typedef struct grab grab;
-struct grab
+static long pathconf_inner _((char *, int));
+
+static long
+pathconf_inner(path, arg)
+	char		*path;
+	int		arg;
 {
-	double	g_elapsed;
-	double	g_cpu;
-	double	g_io;
-};
+	long		result;
 
-static void os_meter_grab _((grab *));
-
-static void
-os_meter_grab(gp)
-	grab		*gp;
-{
-#ifdef HAVE_GETRUSAGE
-	struct rusage	ru;
-	struct timeval	tv;
-	struct timezone	tz;
-
-	int getrusage _((int, struct rusage *));
-	int gettimeofday _((struct timeval *, struct timezone *));
-
-	if (getrusage(RUSAGE_CHILDREN, &ru))
-		nfatal("getrusage");
-	if (gettimeofday(&tv, &tz))
-		nfatal("gettimeofday");
-	gp->g_cpu = ru.ru_utime.tv_sec + ru.ru_utime.tv_usec * 1.0e-6;
-	gp->g_io = ru.ru_stime.tv_sec + ru.ru_stime.tv_usec * 1.0e-6;
-	gp->g_elapsed = tv.tv_sec + tv.tv_usec * 1.0e-6;
-#else
-	struct tms	buffer;
-	time_t		n;
-
-	n = times(&buffer);
-	gp->g_elapsed = n / (double)HZ;
-	gp->g_cpu = buffer.tms_cutime / (double)HZ;
-	gp->g_io = buffer.tms_cstime / (double)HZ;
-#endif
-}
-
-
-/*
- * NAME
- *	os_meter_ptime - print timing info
- *
- * SYNOPSIS
- *	void os_meter_ptime(double t, char *s, double elapsed);
- *
- * DESCRIPTION
- *	The os_meter_ptime function is used to print e representation of the
- *	time, in seconds, given in the `t' argument.  If `elapsed' is >0, it is
- *	the elapsed time taken, for percentage figures.  The `s' argument is a
- *	title string.
- *
- * RETURNS
- *	void
- */
-
-static void os_meter_ptime _((double, char *, double));
-
-static void
-os_meter_ptime(t,s,e)
-	double	t;
-	char	*s;
-	double	e;
-{
-	char	buffer[32];
-	char	temp[32];
-	long	hour, min, sec, frac;
-
-	frac = t * 1000 + 0.5;
-	sec = frac / 1000;
-	frac %= 1000;
-	min = sec / 60;
-	sec %= 60;
-	hour = min / 60;
-	min %= 60;
-	if (e > 0.0)
+	errno = EINVAL; /* IRIX 5.2 fails to set on errors */
+	result = pathconf(path, arg);
+	if (result < 0)
 	{
-		sprintf(temp, "(%.2f%%)", 100. * t / e);
-		sprintf(buffer, "%-4s%9s", s, temp);
-		s = buffer;
+		switch (errno)
+		{
+		case ENOSYS: /* lotsa systems say this for EINVAL */
+#ifdef EOPNOTSUPP
+		case EOPNOTSUPP: /* HPUX says this for EINVAL */
+#endif
+			errno = EINVAL;
+			break;
+		}
 	}
-	fprintf(stderr, "%2ld:%02ld:%02ld.%03ld %s\n", hour, min, sec, frac, s);
+	return result;
 }
 
 
-/*
- * NAME
- *	os_meter_begin - start metering interval
- *
- * SYNOPSIS
- *	void os_meter_begin(void);
- *
- * DESCRIPTION
- *	The os_meter_begin function is used to start a metring interval.
- *
- * RETURNS
- *	void
- */
+static long pathconf_wrapper _((char *, int, long));
 
-static	grab	os_meter_start;
-
-void
-os_meter_begin()
+static long
+pathconf_wrapper(path, arg, default_value)
+	char		*path;
+	int		arg;
+	long		default_value;
 {
-	os_meter_grab(&os_meter_start);
+	long		result;
+
+	result = pathconf_inner(path, arg);
+	if (result < 0 && errno == EINVAL)
+	{
+		/*
+		 * probably NFSv2 mounted
+		 * assume is same as root
+		 */
+		result = pathconf_inner("/", arg);
+		if (result < 0 && errno == EINVAL)
+			result = default_value;
+	}
+	return result;
 }
 
-
-/*
- * NAME
- *	os_meter_end - end a metering interval
- *
- * SYNOPSIS
- *	void os_meter_end(void);
- *
- * DESCRIPTION
- *	The os_meter_end function is used to end a metering interval and print
- *	the metered information.
- *
- * RETURNS
- *	void
- */
-
-void
-os_meter_end()
-{
-	grab	end;
-	double	elapsed;
-	double	cpu;
-	double	io;
-
-	os_meter_grab(&end);
-
-	elapsed = end.g_elapsed - os_meter_start.g_elapsed;
-	os_meter_ptime(elapsed, "elapsed", 0.0);
-	cpu = end.g_cpu - os_meter_start.g_cpu;
-	os_meter_ptime(cpu, "usr", elapsed);
-	io = end.g_io - os_meter_start.g_io;
-	os_meter_ptime(io, "sys", elapsed);
-}
+#endif
 
 
 static int os_pathconf_path_max _((char *));
@@ -1285,23 +1340,23 @@ os_pathconf_path_max(path)
 {
 	long	result;
 
-#ifdef _PC_PATH_MAX
-	result = pathconf(path, _PC_PATH_MAX);
-	if (result < 0 && (errno == EINVAL || errno == ENOSYS))
-	{
-		/*
-		 * probably NFSv2 mounted
-		 * assume is same as root
-		 */
-		path = "/";
-		result = pathconf(path, _PC_PATH_MAX);
-		if (result < 0 && (errno == EINVAL || errno == ENOSYS))
-			result = 1024;
-	}
-	if (result < 0)
-		nfatal("pathconf(\"%s\", {PATH_MAX})", path);
-#else
 	result = 1024;
+#ifdef HAVE_PATHCONF
+	result = pathconf_wrapper(path, _PC_PATH_MAX, result);
+	if (result < 0)
+	{
+		sub_context_ty	*scp;
+
+		scp = sub_context_new();
+		sub_errno_set(scp);
+		sub_var_set(scp, "File_Name", "%s", path);
+		fatal_intl
+		(
+			scp,
+			i18n("pathconf(\"$filename\", {PATH_MAX}): $errno")
+		);
+		/* NOTREACHED */
+	}
 #endif
 	return result;
 }
@@ -1315,33 +1370,27 @@ os_pathconf_name_max(path)
 {
 	long	result;
 
-#ifdef _PC_NAME_MAX
-	result = pathconf(path, _PC_NAME_MAX);
-	if (result < 0 && (errno == EINVAL || errno == ENOSYS))
-	{
-		/*
-		 * probably NFSv2 mounted
-		 * assume is same as root
-		 */
-		path = "/";
-		result = pathconf(path, _PC_NAME_MAX);
-		if (result < 0 && (errno == EINVAL || errno == ENOSYS))
-		{
-#ifdef HAVE_LONG_FILE_NAMES
-			result = 255;
-#else
-			result = 14;
-#endif
-		}
-	}
-	if (result < 0)
-		nfatal("pathconf(\"%s\", {NAME_MAX})", path);
-#else
 #ifdef HAVE_LONG_FILE_NAMES
 	result = 255;
 #else
 	result = 14;
 #endif
+#ifdef HAVE_PATHCONF
+	result = pathconf_wrapper(path, _PC_NAME_MAX, result);
+	if (result < 0)
+	{
+		sub_context_ty	*scp;
+
+		scp = sub_context_new();
+		sub_errno_set(scp);
+		sub_var_set(scp, "File_Name", "%s", path);
+		fatal_intl
+		(
+			scp,
+			i18n("pathconf(\"$filename\", {NAME_MAX}): $errno")
+		);
+		/* NOTREACHED */
+	}
 #endif
 	return result;
 }
@@ -1387,38 +1436,59 @@ os_legal_path(str)
 }
 
 
-#ifdef TIMING
-
-static long os_totals_baseline;
-
-void
-os_totals_start()
+int
+os_mkdir(p, echo)
+	string_ty	*p;
+	int		echo;
 {
-	struct tms	buffer;
+	char		*cp;
 
-	os_totals_baseline = times(&buffer);
+	for (cp = p->str_text; ; ++cp)
+	{
+		if (cp > p->str_text && (*cp == '/' || *cp == 0))
+		{
+			string_ty	*tmp;
+
+			tmp = str_n_from_c(p->str_text, cp - p->str_text);
+			if (mkdir(tmp->str_text, 0777) < 0)
+			{
+				if (errno != EEXIST)
+				{
+					sub_context_ty	*scp;
+
+					scp = sub_context_new();
+					sub_errno_set(scp);
+					sub_var_set(scp, "File_Name", "%S", tmp);
+					error_intl
+					(
+						scp,
+						i18n("mkdir $filename: $errno")
+					);
+					sub_context_delete(scp);
+					str_free(tmp);
+					option_set_errors();
+					return -1;
+				}
+			}
+			else if (echo)
+			{
+				sub_context_ty	*scp;
+
+				scp = sub_context_new();
+				sub_var_set(scp, "File_Name", "%S", tmp);
+				error_intl(scp, i18n("mkdir $filename"));
+				sub_context_delete(scp);
+			}
+
+			/*
+			 * if the knew about the existence of the file
+			 * before we deleted it, then we will have to
+			 * adjust the stat cache.
+			 */
+			stat_cache_clear(tmp);
+			str_free(tmp);
+		}
+		if (*cp == 0)
+			return 0;
+	}
 }
-
-void
-os_totals()
-{
-	struct tms	buffer;
-	time_t		n;
-	double		elapsed;
-	double		t;
-
-	error("timing totals...");
-	n = times(&buffer);
-	elapsed = (n - os_totals_baseline) / (double)HZ;
-	os_meter_ptime(elapsed, "elapsed", 0.0);
-	t = buffer.tms_utime / (double)HZ;
-	os_meter_ptime(t, "usr", elapsed);
-	t = buffer.tms_stime / (double)HZ;
-	os_meter_ptime(t, "sys", elapsed);
-	t = buffer.tms_cutime / (double)HZ;
-	os_meter_ptime(t, "cusr", elapsed);
-	t = buffer.tms_cstime / (double)HZ;
-	os_meter_ptime(t, "csys", elapsed);
-}
-
-#endif
