@@ -1,6 +1,6 @@
 /*
  *	cook - file construction tool
- *	Copyright (C) 1992, 1993, 1994 Peter Miller.
+ *	Copyright (C) 1992, 1993, 1994, 1997, 1998 Peter Miller;
  *	All rights reserved.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -15,7 +15,7 @@
  *
  *	You should have received a copy of the GNU General Public License
  *	along with this program; if not, write to the Free Software
- *	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
  *
  * MANIFEST: operating system start point, and parse command line arguments
  */
@@ -26,18 +26,18 @@
 #include <ac/stdlib.h>
 
 #include <sys/types.h>
-#include <utime.h>
+#include <ac/utime.h>
 #include <sys/stat.h>
 
 #include <arglex.h>
 #include <date.h>
-#include <error.h>
+#include <error_intl.h>
 #include <help.h>
 #include <main.h>
+#include <progname.h>
 #include <str.h>
-#include <trace.h>
 #include <version.h>
-#include <word.h>
+#include <str_list.h>
 
 
 static void usage _((void));
@@ -45,6 +45,9 @@ static void usage _((void));
 static void
 usage()
 {
+	char		*progname;
+
+	progname = progname_get();
 	fprintf(stderr, "usage: %s [ <option>... ] <filename>\n", progname);
 	fprintf(stderr, "       %s -Help\n", progname);
 	fprintf(stderr, "       %s -VERsion\n", progname);
@@ -52,22 +55,6 @@ usage()
 }
 
 
-static void main_help _((void));
-
-static void
-main_help()
-{
-	static char *text[] =
-	{
-#include <../man1/cooktime.h>
-	};
-
-	trace(("c_incl_help()\n{\n"/*}*/));
-	help(text, SIZEOF(text), usage);
-	trace((/*{*/"}\n"));
-}
-
-
 enum
 {
 	arglex_token_access,
@@ -91,7 +78,7 @@ main(argc, argv)
 	int		argc;
 	char		**argv;
 {
-	wlist		filename;
+	string_list_ty	filename;
 	string_ty	*s;
 	size_t		j;
 	int		mtime_set;
@@ -105,7 +92,7 @@ main(argc, argv)
 	switch (arglex())
 	{
 	case arglex_token_help:
-		main_help();
+		help((char *)0, usage);
 		exit(0);
 
 	case arglex_token_version:
@@ -121,34 +108,30 @@ main(argc, argv)
 	atime = 0;
 	mtime_set = 0;
 	mtime = 0;
-	wl_zero(&filename);
+	string_list_constructor(&filename);
 	while (arglex_token != arglex_token_eoln)
 	{
 		switch(arglex_token)
 		{
 		default:
-			error
-			(
-				"misplaced \"%s\" command line argument",
-				arglex_value.alv_string
-			);
-			usage();
+			generic_argument(usage);
+			continue;
 
 		case arglex_token_string:
 			s = str_from_c(arglex_value.alv_string);
-			wl_append(&filename, s);
+			string_list_append(&filename, s);
 			str_free(s);
 			break;
 
 		case arglex_token_report:
 			if (report)
 			{
+				sub_context_ty	*scp;
+
 				duplicate:
-				fatal
-				(
-					"duplicate \"%s\" option",
-					arglex_value.alv_string
-				);
+				scp = sub_context_new();
+				sub_var_set(scp, "Name", "%s", arglex_value.alv_string);
+				fatal_intl(scp, i18n("duplicate \"$name\" option"));
 			}
 			report++;
 			break;
@@ -157,63 +140,81 @@ main(argc, argv)
 			if (atime_set)
 				goto duplicate;
 			if (arglex() != arglex_token_string)
-				fatal("-Access requires a string argument");
+			{
+				sub_context_ty	*scp;
+
+				scp = sub_context_new();
+				sub_var_set(scp, "Name", "%s", arglex_token_name(arglex_token_access));
+				fatal_intl(scp, i18n("$name needs string"));
+			}
 			atime_set = 1;
 			atime = date_scan(arglex_value.alv_string);
 			if (atime < 0)
-				fatal
+			{
+				sub_context_ty	*scp;
+
+				scp = sub_context_new();
+				sub_var_set(scp, "Value", "%s", arglex_value.alv_string);
+				fatal_intl
 				(
-					"the string \"%s\" is not a valid date",
-					arglex_value.alv_string
+					scp,
+			       i18n("the string \"$value\" is not a valid date")
 				);
+			}
 			break;
 
 		case arglex_token_modify:
 			if (mtime_set)
 				goto duplicate;
 			if (arglex() != arglex_token_string)
-				fatal("-Modify requires a string argument");
+			{
+				sub_context_ty	*scp;
+
+				scp = sub_context_new();
+				sub_var_set(scp, "Name", "%s", arglex_token_name(arglex_token_modify));
+				fatal_intl(scp, i18n("$name needs string"));
+			}
 			mtime_set = 1;
 			mtime = date_scan(arglex_value.alv_string);
 			if (mtime < 0)
-				fatal
-				(
-					"the string \"%s\" is not a valid date",
-					arglex_value.alv_string
-				);
-			break;
-
-#ifdef DEBUG
-		case arglex_token_tracing:
-			if (arglex() != arglex_token_string)
-				fatal("-TRACIng requires one or more string arguments");
-			for (;;)
 			{
-				trace_enable(arglex_value.alv_string);
-				if (arglex() != arglex_token_string)
-					break;
+				sub_context_ty	*scp;
+
+				scp = sub_context_new();
+				sub_var_set(scp, "Value", "%s", arglex_value.alv_string);
+				fatal_intl
+				(
+					scp,
+			       i18n("the string \"$value\" is not a valid date")
+				);
 			}
-			continue;
-#endif
+			break;
 		}
 		arglex();
 	}
-	if (!filename.wl_nwords)
-		fatal("no files named");
+	if (!filename.nstrings)
+		fatal_intl(0, i18n("no files named"));
 	if (!report && !mtime_set && !atime_set)
 	{
 		mtime_set = 1;
 		mtime = date_scan("now");
 	}
 
-	for (j = 0; j < filename.wl_nwords; ++j)
+	for (j = 0; j < filename.nstrings; ++j)
 	{
 		struct stat	st;
 		struct utimbuf	ut;
 
-		s = filename.wl_word[j];
+		s = filename.string[j];
 		if (stat(s->str_text, &st))
-			nfatal("%s", s->str_text);
+		{
+			sub_context_ty	*scp;
+
+			scp = sub_context_new();
+			sub_errno_set(scp);
+			sub_var_set(scp, "File_Name", "%S", s);
+			fatal_intl(scp, i18n("stat $filename: $errno"));
+		}
 		if (mtime_set)
 			ut.modtime = mtime;
 		else
@@ -223,7 +224,13 @@ main(argc, argv)
 		else
 			ut.actime = st.st_atime;
 		if ((atime_set || mtime_set) && utime(s->str_text, &ut))
-			nfatal("%s", s->str_text);
+		{
+			sub_context_ty	*scp;
+
+			scp = sub_context_new();
+			sub_var_set(scp, "File_Name", "%S", s);
+			fatal_intl(scp, i18n("utime $filename: $errno"));
+		}
 		if (report)
 		{
 			printf("%s\n", s->str_text);
@@ -231,6 +238,13 @@ main(argc, argv)
 			printf("\tmodify %s\n", date_string(ut.modtime));
 		}
 	}
+#if defined(sun) || defined(__sun__)
+	/*
+	 * Solaris is brain dead.  The inode times are not updated until
+	 * the next sync.  So we'll help it along.
+	 */
+	sync();
+#endif
 	exit(0);
 	return 0;
 }
