@@ -1,6 +1,6 @@
 /*
  *	cook - file construction tool
- *	Copyright (C) 1990, 1991, 1992, 1993 Peter Miller.
+ *	Copyright (C) 1990, 1991, 1992, 1993, 1997, 1998 Peter Miller;
  *	All rights reserved.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -15,7 +15,7 @@
  *
  *	You should have received a copy of the GNU General Public License
  *	along with this program; if not, write to the Free Software
- *	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
  *
  * MANIFEST: functions to isolate operating system interface
  */
@@ -24,7 +24,7 @@
 #include <sys/stat.h>
 #include <errno.h>
 
-#include <error.h>
+#include <error_intl.h>
 #include <os.h>
 
 /*
@@ -40,8 +40,9 @@
 
 int
 os_exists(filename)
-	char	*filename;
+	char		*filename;
 {
+	sub_context_ty	*scp;
 	struct stat	st;
 
 	if (stat(filename, &st))
@@ -53,7 +54,11 @@ os_exists(filename)
 			break;
 
 		default:
-			nfatal("%s", filename);
+			scp = sub_context_new();
+			sub_errno_set(scp);
+			sub_var_set(scp, "File_Name", "%s", filename);
+			fatal_intl(scp, i18n("stat $filename: $errno"));
+			/* NOTREACHED */
 		}
 		return 0;
 	}
