@@ -1,6 +1,6 @@
 /*
  *	cook - file construction tool
- *	Copyright (C) 1994 Peter Miller.
+ *	Copyright (C) 1994, 1997, 1998 Peter Miller;
  *	All rights reserved.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -15,7 +15,7 @@
  *
  *	You should have received a copy of the GNU General Public License
  *	along with this program; if not, write to the Free Software
- *	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
  *
  * MANIFEST: functions to emit the result
  */
@@ -23,56 +23,87 @@
 #include <stdio.h>
 
 #include <emit.h>
-#include <error.h>
+#include <error_intl.h>
+#include <trace.h>
 
 static string_ty *filename;
 static FILE	*output;
 int		emit_line_numbers;
-int		col;
-long		old_ln;
-string_ty	*old_fn;
-int		depth;
+static int	icol;
+static int	ocol;
+static long	old_ln;
+static string_ty *old_fn;
+static int	depth;
+static int	back_slash_seen;
 
 
 void
 emit_open(s)
 	char		*s;
 {
+	sub_context_ty	*scp;
+
+	trace(("open\n"));
 	assert(!output);
 	if (s)
 	{
 		output = fopen(s, "w");
 		if (!output)
-			nfatal("%s", s);
+		{
+			scp = sub_context_new();
+			sub_errno_set(scp);
+			sub_var_set(scp, "File_Name", "%s", s);
+			fatal_intl(scp, i18n("open $filename: $errno"));
+		}
 		filename = str_from_c(s);
 	}
 	else
 	{
-		filename = str_from_c("standard output");
+		scp = sub_context_new();
+		filename = subst_intl(scp, i18n("standard output"));
+		sub_context_delete(scp);
 		output = stdout;
 	}
-	col = 0;
+	icol = 0;
+	ocol = 0;
+	back_slash_seen = 0;
 }
 
 
 void
 emit_close()
 {
+	sub_context_ty	*scp;
+
 	assert(output);
 	emit_bol();
+	if (fflush(output))
+	{
+		scp = sub_context_new();
+		sub_errno_set(scp);
+		sub_var_set(scp, "File_Name", "%S", filename);
+		fatal_intl(scp, i18n("write $filename: $errno"));
+	}
 	if (output != stdout && fclose(output))
-		nfatal("%s", filename->str_text);
+	{
+		scp = sub_context_new();
+		sub_errno_set(scp);
+		sub_var_set(scp, "File_Name", "%S", filename);
+		fatal_intl(scp, i18n("close $filename: $errno"));
+	}
 	str_free(filename);
 	filename = 0;
 	output = 0;
-	col = 0;
+	icol = 0;
+	ocol = 0;
+	back_slash_seen = 0;
 }
 
 
 void
 emit_bol()
 {
-	if (col)
+	if (ocol)
 		emit_char('\n');
 }
 
@@ -81,24 +112,58 @@ void
 emit_char(c)
 	int		c;
 {
-	if (c != '\n' && !col && depth > 0)
+	if (!icol && !ocol)
+		icol = 8 * depth;
+	if (back_slash_seen && (c == ' ' || c == '\\'))
 	{
-		int	j;
+		assert(icol == ocol);
+		back_slash_seen = 0;
+		goto normal;
+	}
+	back_slash_seen = (c == '\\');
+	switch (c)
+	{
+	case ' ':
+		++icol;
+		return;
 
-		for (j = 0; j < depth; ++j)
-			putc('\t', output);
+	case '\t':
+		icol = (icol + 8) & ~7;
+		return;
+
+	case '\n':
+		if (putc(c, output) == EOF)
+			goto bomb;
+		icol = 0;
+		ocol = 0;
+		++old_ln;
+		return;
+	}
+	normal:
+	while (((ocol + 8) & ~7) <= icol && ocol + 1 < icol)
+	{
+		if (putc('\t', output) == EOF)
+			goto bomb;
+		ocol = (ocol + 8) & ~7;
+	}
+	while (ocol < icol)
+	{
+		if (putc(' ', output) == EOF)
+			goto bomb;
+		++ocol;
 	}
 	if (putc(c, output) == EOF)
-		nfatal("%s", filename->str_text);
-	if (c == '\n')
 	{
-		++old_ln;
-		col = 0;
-	}
-	else
-	{
-		++col;
+		sub_context_ty	*scp;
+
+		bomb:
+		scp = sub_context_new();
+		sub_errno_set(scp);
+		sub_var_set(scp, "File_Name", "%S", filename);
+		fatal_intl(scp, i18n("write $filename: $errno"));
 	}
+	++icol;
+	++ocol;
 }
 
 
@@ -150,9 +215,16 @@ emit_line_number(lino, fn)
 		emit_bol();
 		fprintf(output, "#line %ld \"%s\"\n", lino, fn->str_text);
 		if (ferror(output))
-			nfatal("write %s", filename->str_text);
+		{
+			sub_context_ty	*scp;
+
+			scp = sub_context_new();
+			sub_errno_set(scp);
+			sub_var_set(scp, "File_Name", "%S", filename);
+			fatal_intl(scp, i18n("write $filename: $errno"));
+		}
 	}
-	else if (!str_equal(old_fn, fn) || lino > (old_ln + !!col))
+	else if (!str_equal(old_fn, fn) || lino > (old_ln + (ocol != 0)))
 	{
 		emit_bol();
 		emit_char('\n');
