@@ -1,7 +1,7 @@
 #!/bin/sh
 #
 #	cook - file construction tool
-#	Copyright (C) 1994 Peter Miller.
+#	Copyright (C) 1994, 1997, 1998 Peter Miller;
 #	All rights reserved.
 #
 #	This program is free software; you can redistribute it and/or modify
@@ -16,11 +16,11 @@
 #
 #	You should have received a copy of the GNU General Public License
 #	along with this program; if not, write to the Free Software
-#	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
 #
 # MANIFEST: Test the fingerprint short-circuit functionality
 #
-work=/tmp/$$
+work=${COOK_TMP:-/tmp}/$$
 PAGER=cat
 export PAGER
 umask 022
@@ -28,7 +28,7 @@ unset COOK
 here=`pwd`
 if test $? -ne 0 ; then exit 1; fi
 
-if test "$1" != "" ; then bin="$here/$1/bin"; else bin="$here/bin"; fi
+bin="$here/${1-.}/bin"
 
 fail()
 {
@@ -47,11 +47,19 @@ pass()
 }
 trap "fail" 1 2 3 15
 
-mkdir $work
+mkdir $work $work/lib
 if test $? -ne 0 ; then exit 1; fi
 cd $work
 if test $? -ne 0 ; then fail; fi
 
+#
+# Use the default error messages.  There is no other way to get
+# predictable test behaviour on the unknown systems we will be tested on.
+#
+COOK_MESSAGE_LIBRARY=$work/no-such-dir
+export COOK_MESSAGE_LIBRARY
+unset LANG
+
 #
 # this test is to see of the file "bar" is correctly fingerprinted.
 # The second time the recipe is applied, it will fail.
